<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateAdminNotificationAPIRequest;
use App\Http\Requests\API\UpdateAdminNotificationAPIRequest;
use App\Models\AdminNotification;
use App\Models\User;
use App\Repositories\AdminNotificationRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\AdminNotificationResource;
use Response;

/**
 * Class AdminNotificationController
 * @package App\Http\Controllers\API
 */

class AdminNotificationAPIController extends AppBaseController
{
    /** @var  AdminNotificationRepository */
    private $adminNotificationRepository;

    public function __construct(AdminNotificationRepository $adminNotificationRepo)
    {
        $this->adminNotificationRepository = $adminNotificationRepo;
    }

    /**
     * Display a listing of the AdminNotification.
     * GET|HEAD /adminNotifications
     *
     * @param Request $request
     * @return Response
     */
    public function index(Request $request)
    {
        $adminNotifications = $this->adminNotificationRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            AdminNotificationResource::collection($adminNotifications),
            __('messages.retrieved', ['model' => __('models/adminNotifications.plural')])
        );
    }

    /**
     * Store a newly created AdminNotification in storage.
     * POST /adminNotifications
     *
     * @param CreateAdminNotificationAPIRequest $request
     *
     * @return Response
     */
    public function store(CreateAdminNotificationAPIRequest $request)
    {
        $input = $request->all();

        if($request->file('image')){
            $input['image'] = $this->saveImage($request->file('image'), 'adminNotifications');
        }


        $adminNotification = $this->adminNotificationRepository->create($input);

        if($request->users){
            if($request->users == 'all_students'){
                $users = User::where('role','student')->pluck('id')->toArray();
            }
            elseif($request->users == 'all_trainers'){
                $users = User::where('role','trainer')->pluck('id')->toArray();
            }
            else{
                $users = $request->users;
            }
            foreach ($users as $user_id){
                $user = User::find($user_id);
                if($user){
                    $user->notify(new \App\Notifications\Send($adminNotification));

                    if($user->token){
                        sendFirebaseNotification($user->token,$adminNotification->title,$adminNotification->desc);
                    }
                }
            }

        }
        return $this->sendResponse(
            new AdminNotificationResource($adminNotification),
            __('messages.saved', ['model' => __('models/adminNotifications.singular')])
        );
    }

    /**
     * Display the specified AdminNotification.
     * GET|HEAD /adminNotifications/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var AdminNotification $adminNotification */
        $adminNotification = $this->adminNotificationRepository->find($id);

        if (empty($adminNotification)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/adminNotifications.singular')])
            );
        }

        return $this->sendResponse(
            new AdminNotificationResource($adminNotification),
            __('messages.retrieved', ['model' => __('models/adminNotifications.singular')])
        );
    }

    /**
     * Update the specified AdminNotification in storage.
     * PUT/PATCH /adminNotifications/{id}
     *
     * @param int $id
     * @param UpdateAdminNotificationAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateAdminNotificationAPIRequest $request)
    {
        $input = $request->all();

        /** @var AdminNotification $adminNotification */
        $adminNotification = $this->adminNotificationRepository->find($id);

        if (empty($adminNotification)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/adminNotifications.singular')])
            );
        }

        $input['image'] = $request->hasFile('image') ?$this->updateImage($adminNotification->image,$request->file('image'), 'adminNotifications'):$adminNotification->image;
        $adminNotification = $this->adminNotificationRepository->update($input, $id);

        return $this->sendResponse(
            new AdminNotificationResource($adminNotification),
            __('messages.updated', ['model' => __('models/adminNotifications.singular')])
        );
    }

    /**
     * Remove the specified AdminNotification from storage.
     * DELETE /adminNotifications/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var AdminNotification $adminNotification */
        $adminNotification = $this->adminNotificationRepository->find($id);

        if (empty($adminNotification)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/adminNotifications.singular')])
            );
        }

        $adminNotification->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/adminNotifications.singular')])
        );
    }
}
