<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateAgeCategoryAPIRequest;
use App\Http\Requests\API\UpdateAgeCategoryAPIRequest;
use App\Models\AgeCategory;
use App\Repositories\AgeCategoryRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\AgeCategoryResource;
use Response;

/**
 * Class AgeCategoryController
 * @package App\Http\Controllers\API
 */

class AgeCategoryAPIController extends AppBaseController
{
    /** @var  AgeCategoryRepository */
    private $ageCategoryRepository;

    public function __construct(AgeCategoryRepository $ageCategoryRepo)
    {
        $this->ageCategoryRepository = $ageCategoryRepo;
    }

    /**
     * Display a listing of the AgeCategory.
     * GET|HEAD /ageCategories
     *
     * @param Request $request
     * @return Response
     */
    public function index(Request $request)
    {
        $ageCategories = $this->ageCategoryRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            AgeCategoryResource::collection($ageCategories),
            __('messages.retrieved', ['model' => __('models/ageCategories.plural')])
        );
    }

    /**
     * Store a newly created AgeCategory in storage.
     * POST /ageCategories
     *
     * @param CreateAgeCategoryAPIRequest $request
     *
     * @return Response
     */
    public function store(CreateAgeCategoryAPIRequest $request)
    {
        $input = $request->all();

        $ageCategory = $this->ageCategoryRepository->create($input);

        return $this->sendResponse(
            new AgeCategoryResource($ageCategory),
            __('messages.saved', ['model' => __('models/ageCategories.singular')])
        );
    }

    /**
     * Display the specified AgeCategory.
     * GET|HEAD /ageCategories/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var AgeCategory $ageCategory */
        $ageCategory = $this->ageCategoryRepository->find($id);

        if (empty($ageCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/ageCategories.singular')])
            );
        }

        return $this->sendResponse(
            new AgeCategoryResource($ageCategory),
            __('messages.retrieved', ['model' => __('models/ageCategories.singular')])
        );
    }

    /**
     * Update the specified AgeCategory in storage.
     * PUT/PATCH /ageCategories/{id}
     *
     * @param int $id
     * @param UpdateAgeCategoryAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateAgeCategoryAPIRequest $request)
    {
        $input = $request->all();

        /** @var AgeCategory $ageCategory */
        $ageCategory = $this->ageCategoryRepository->find($id);

        if (empty($ageCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/ageCategories.singular')])
            );
        }

        $ageCategory = $this->ageCategoryRepository->update($input, $id);

        return $this->sendResponse(
            new AgeCategoryResource($ageCategory),
            __('messages.updated', ['model' => __('models/ageCategories.singular')])
        );
    }

    /**
     * Remove the specified AgeCategory from storage.
     * DELETE /ageCategories/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var AgeCategory $ageCategory */
        $ageCategory = $this->ageCategoryRepository->find($id);

        if (empty($ageCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/ageCategories.singular')])
            );
        }

        $ageCategory->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/ageCategories.singular')])
        );
    }
}
