<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\AppBaseController;
use App\Http\Requests\API\LoginEmailRequest;
use App\Http\Requests\API\LoginRequest;
use App\Http\Requests\API\RegisterRequest;
use App\Http\Requests\API\TrainerRegisterRequest;
use App\Http\Requests\API\UpdateUserAPIRequest;
use App\Http\Resources\AuthResource;
use App\Http\Resources\MyChampionshipResource;
use App\Http\Resources\StatCompetitionResource;
use App\Http\Resources\StatTrainingResource;
use App\Http\Resources\StudentResource;
use App\Http\Resources\TrainerResource;
use App\Http\Resources\TypeLevelResource;
use App\Http\Resources\UserResource;
use App\Models\Championship;
use App\Models\Competition;
use App\Models\Training;
use App\Models\User;
use App\Models\UserLevel;
use App\Repositories\UserRepository;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\Subscription;
use App\Http\Resources\SubscriptionResource;
use Illuminate\Support\Facades\Lang;
use Validator;

use Twilio\Rest\Client;

class AuthAPIController extends AppBaseController
{
    /** @var  UserRepository */
    private UserRepository $userRepository;

    public function __construct(UserRepository $userRepo)
    {
        $this->userRepository = $userRepo;
    }

    public function login(LoginRequest $request): \Illuminate\Http\JsonResponse
    {
        if (Auth::attempt(['phone' => $request->phone, 'password' => $request->password])) {

            $user = Auth::user();
            
        
             if ($request->device_token && $user->device_token
             && $request->device_token != $user->device_token){
                return $this->sendError('You can only login from only one device');
            }
            
            if ($request->role && $request->role != $user->role){
                return $this->sendError(Lang::get('auth.failed').trans('crud.you_are').trans('enum.'.$user->role).trans('crud.not').trans('enum.'.$request->role));
            }
            $user->online = true;
            $user->last_login_at = now();
            $user->save();

            return $this->sendResponse(
                new AuthResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')])
            );
        } else {
            return $this->sendError(Lang::get('auth.failed'));
        }

    }

    public function login_email(LoginEmailRequest $request): \Illuminate\Http\JsonResponse
    {
        if (Auth::attempt(['email' => $request->email, 'password' => $request->password])) {
            $user = Auth::user();
            $user->online = true;
            $user->last_login_at = now();
            $user->save();
            return $this->sendResponse(
                new \App\Http\Resources\admin\AuthResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')])
            );
        } else {
            return $this->sendError(Lang::get('auth.failed'));
        }

    }

    /**
     * Register api
     *
     * @param RegisterRequest $request
     * @return JsonResponse
     */
    public function register(RegisterRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();
        $input['password'] = bcrypt($input['password']);
        $input['role'] = 'student';
        $input['active'] = 1;
        if($request->file('logo')){
            $input['logo'] = $this->saveImage($request->file('logo'), 'student');
        }

        $user = $this->userRepository->create($input);
        $user->online = true;
        $user->last_login_at = now();
        $user->save();

        return $this->sendResponse(
            new AuthResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );

    }

    public function trainer_register(TrainerRegisterRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();
        $input['password'] = bcrypt($input['password']);
        $input['role'] = 'trainer';
        $input['active'] = 1;

        if($request->file('logo')){
            $input['logo'] = $this->saveImage($request->file('logo'), 'trainer');
        }

        $user = $this->userRepository->create($input);
        $user->online = true;
        $user->last_login_at = now();
        $user->save();

        return $this->sendResponse(
            new AuthResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );

    }

    public function verify(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'phone' => 'unique:users,phone'
        ]);

        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first(), $validator->errors());
        }
        $otp = rand(1000, 9999);
        $msg = "DrMath otp: {$otp}";

        if (isset($request->phone)) {
           $phone_number = preg_replace('/^0/',$request->country_code,$request->phone);
    
           $this->send_sms($phone_number, $msg);

            $data =[];
            $data['otp'] = $otp;
            return $this->sendResponse($data,trans('auth.otp.OtpSentSuccessfully'));

        }
    }

public function send_sms($phone, $msg)
{
    $sid = "ACcf0046f7910542b91f58274b5c914f14";
    $token = "2485fbfc4191a63b2b2540d4b89bd96a";
    $fromNumber = "+14423271484";

    try {
        $client = new Client($sid, $token);
        $message = $client->messages->create($phone, [
            'from' => $fromNumber,
            'body' => $msg,
        ]);

        // Log the message SID for debugging
        error_log('Twilio Message SID: ' . $message->sid);

        return true;
    } catch (\Twilio\Exceptions\RestException $e) {
        error_log('Twilio Error: ' . $e->getMessage());
        return 'Error: ' . $e->getMessage();
    }
}

    public function phone_verify(Request $request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'phone' => 'required|exists:users,phone',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first());
        }


        $otp = rand(1000, 9999);
        $msg = "DrMath otp: {$otp}";

         $phone_number = formatPhoneNumber($request->phone);
       
        $this->send_sms($phone_number, $msg);

        $user = User::where('phone',$request->phone)->first();
        if (empty($user)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/users.singular')])
            );
        }
        $user->otp = $otp;
        $user->save();

        $data =[];
        $data['otp'] = $otp;
        return $this->sendResponse($data,trans('auth.otp.OtpSentSuccessfully'));
    }
    public function phone_otp_verify(Request $request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'otp' => 'required||exists:users,otp',
            'phone' => 'required|exists:users,phone',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first());
        }

        $user = User::where('otp', $request->otp)->where('phone',$request->phone)->first();
        if ($user) {
            $user->phone_verified_at = now();
            $user->otp = null;
            return $this->sendResponse(
                new AuthResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')])
            );

        } else {
            return $this->sendError(trans('auth.otp.otp_is_not_correct'));
        }

    }

    public function change_password(Request $request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'password' => 'required',
            'new_password' => 'required|confirmed',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors());
        }

        $user = $request->user();

        if (password_verify($request->password, $user->password)) {
            $user->password = bcrypt($request->new_password);
            $user->save();
            return $this->sendSuccess(trans('auth.Password_changed_Successfully'));
        }
        return $this->sendError(trans('auth.old_password_is_not_correct'));

    }

    public function my_subscription (Request $request): JsonResponse
    {
        $subscription = Subscription::where('user_id',auth('sanctum')->id())->whereDate('end_date','>',now())->orderBy('created_at', 'desc')->first();

        return $this->sendResponse(
            $subscription? new SubscriptionResource($subscription):null,
            __('messages.retrieved', ['model' => __('models/subscriptions.singular')]));

    }

    public function get_profile(Request $request): JsonResponse|bool
    {
        $user = $request->user();
        if ($user->role =="student"){
            return $this->sendResponse(
                new StudentResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')]));
        }
        if ($user->role =="trainer"){
            return $this->sendResponse(
                new TrainerResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')]));
        }
        return $this->sendResponse(
            new UserResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')]));

    }

    public function update_profile(Request $request): JsonResponse
    {
        $user = $request->user();

        $input = $request->all();
        $input['logo'] = $request->file('logo') != null ? $this->updateImage($user->logo , $request->file('logo'), $request->get('type')):$user->logo ;

        $user->update($input);

        return $this->sendResponse(
            new UserResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')]));
    }

    public function update_device_token(Request $request): JsonResponse
    {

        $validator = Validator::make($request->all(), [
            'device_token' => 'required',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors());
        }

        $user = $request->user();
        $user->token = $request->device_token;
        $user->save();

        return $this->sendSuccess(trans('auth.TokenUpdatedSuccessfully'));
    }

    public function update_user_levels(Request $request): JsonResponse
    {
        $user = $request->user();
        if ($request->levels){
            $user->levels()->sync($request->levels);
            $user->save();
        }

        return $this->sendResponse(
            TypeLevelResource::collection($user->levels),
            __('messages.retrieved', ['model' => __('models/typeLevels.plural')])
        );
    }

    public function update_user_online(Request $request): JsonResponse
    {
        $user = $request->user();
        $user->online = !$user->online;
        $user->save();

        return $this->sendSuccess(trans('auth.StatusUpdatedSuccessfully'));
    }


    public function logout(Request $request): JsonResponse
    {

        $user = $request->user();
        $user->device_token = null;
        $user->online = false;
        $user->save();

        if (auth('sanctum')->check()) {
            $user->currentAccessToken()->delete();
        }

        return $this->sendSuccess(trans('auth.LogoutSuccessfully'));
    }

    public function delete_account(): JsonResponse
    {
        if (auth('sanctum')->check()) {
            $user = User::find( auth('sanctum')->id());
            $user->online = false;
            $user->save();
            $user->delete();
        }

        return $this->sendSuccess(trans('auth.AccountDeletedSuccessfully'));
    }

    public function delete_logo(): JsonResponse
    {
        if (auth('sanctum')->check()) {
            $user = User::find( auth('sanctum')->id());
            $this->fileRemove($user->logo);
            $user->logo = null;
            $user->save();
        }

        return $this->sendSuccess(trans('auth.LogoDeletedSuccessfully'));
    }

    public function social_login(Request $request): JsonResponse
    {
        $user= User::where('profile_id', $request->input('profile_id'))->first();

        if(empty($user)){
            $pass = '';
            $user= User::create([
                'name' => $request->input('name'),
                'email' => $request->input('email'),
                'phone' => $request->input('phone'),
                'role' => $request->input('role'),
                'profile_id' => $request->input('profile_id'),
                'provide_name'=>$request->input('provide_name'),
                'password'=>bcrypt($pass),
                'country_id' => 1,
                'city_id' => 1,
                'region_id' => 1,

            ]);

        }

        Auth::login($user, true);

        $user->online = true;
        $user->last_login_at = now();
        $user->save();

        return $this->sendResponse(
            new AuthResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );

    }

    public function forget_password(Request $request): JsonResponse
    {
      return $this->phone_verify($request);
    }

    public function password_phone_otp(Request $request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'otp' => 'required||exists:users,otp',
            'phone' => 'required|exists:users,phone',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first());
        }

        $user = User::where('otp', $request->otp)->where('phone',$request->phone)->first();
        if ($user) {
            $user->otp = null;
            return $this->sendResponse(
                new AuthResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')])
            );

        } else {
            return $this->sendError(trans('auth.otp.otp_is_not_correct'));
        }

    }

    public function password_reset(Request $request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'otp' => 'required||exists:users,otp',
            'phone' => 'required|exists:users,phone',
            'new_password' => 'required|confirmed',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first(),$validator->errors());
        }

        $user = User::where('otp', $request->otp)->where('phone',$request->phone)->first();
        if ($user) {
            $user->password = bcrypt($request->get('new_password'));
            $user->save();
            return $this->sendResponse(
                new AuthResource($user),
                __('messages.retrieved', ['model' => __('models/users.singular')])
            );
        } else {
            return $this->sendError(trans('auth.otp.otp_is_not_correct'));
        }

    }

    public function statistics(): JsonResponse
    {
        $data = [];

        $trainings = Training::where('finish',1)->where('user_id',auth('sanctum')->id())->get();

        $competitions = Competition::where('finish',1)->where('user_id',auth('sanctum')->id())->orWhereHas('subscribers', fn ($query) => $query->where('user_id', auth('sanctum')->id()))->get();

        $championships = Championship::where('finish',1)->whereHas('subscribers', fn ($query) => $query->where('user_id', auth('sanctum')->id()))->get();

        if (request('date_from')&&request('date_to') ){
            $from = date(request('date_from'));
            $to = date(request('date_to'));

            $trainings = $trainings->whereBetween('created_at', [$from, $to]);
            $competitions = $competitions->whereBetween('competition_datetime', [$from, $to]);
            $championships = $championships->whereBetween('start_at', [$from, $to]);
        }

        $data['trainings_count'] = count($trainings);
        $data['competitions_count'] = count($competitions);
        $data['championships_count'] = count($championships);


        $data['trainings'] =  StatTrainingResource::collection($trainings);
        $data['competitions'] =  StatCompetitionResource::collection($competitions);
        $data['championships'] =  MyChampionshipResource::collection($championships);

        return $this->sendResponse(
            $data,
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );

    }


}
