<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateChampionshipMessageAPIRequest;
use App\Http\Requests\API\UpdateChampionshipMessageAPIRequest;
use App\Models\ChampionshipMessage;
use App\Repositories\ChampionshipMessageRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\ChampionshipMessageResource;
use Response;

/**
 * Class ChampionshipMessageController
 * @package App\Http\Controllers\API
 */

class ChampionshipMessageAPIController extends AppBaseController
{
    /** @var  ChampionshipMessageRepository */
    private $championshipMessageRepository;

    public function __construct(ChampionshipMessageRepository $championshipMessageRepo)
    {
        $this->championshipMessageRepository = $championshipMessageRepo;
    }

    /**
     * Display a listing of the ChampionshipMessage.
     * GET|HEAD /championshipMessages
     *
     * @param Request $request
     * @return Response
     */
    public function index(Request $request)
    {
        $championshipMessages = $this->championshipMessageRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('championship_id',$request->championship_id);

        return $this->sendResponse(
            ChampionshipMessageResource::collection($championshipMessages),
            __('messages.retrieved', ['model' => __('models/championshipMessages.plural')])
        );
    }

    /**
     * Store a newly created ChampionshipMessage in storage.
     * POST /championshipMessages
     *
     * @param CreateChampionshipMessageAPIRequest $request
     *
     * @return Response
     */
    public function store(CreateChampionshipMessageAPIRequest $request)
    {
        $input = $request->all();
        $input['user_id'] = auth('sanctum')->id();

        if($request->file('file')){
            $input['file'] = $this->saveImage($request->file('file'), 'championshipMessages');
        }

        $championshipMessage = $this->championshipMessageRepository->create($input);

        return $this->sendResponse(
            new ChampionshipMessageResource($championshipMessage),
            __('messages.saved', ['model' => __('models/championshipMessages.singular')])
        );
    }

    /**
     * Display the specified ChampionshipMessage.
     * GET|HEAD /championshipMessages/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var ChampionshipMessage $championshipMessage */
        $championshipMessage = $this->championshipMessageRepository->find($id);

        if (empty($championshipMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipMessages.singular')])
            );
        }

        return $this->sendResponse(
            new ChampionshipMessageResource($championshipMessage),
            __('messages.retrieved', ['model' => __('models/championshipMessages.singular')])
        );
    }

    /**
     * Update the specified ChampionshipMessage in storage.
     * PUT/PATCH /championshipMessages/{id}
     *
     * @param int $id
     * @param UpdateChampionshipMessageAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateChampionshipMessageAPIRequest $request)
    {
        $input = $request->all();

        /** @var ChampionshipMessage $championshipMessage */
        $championshipMessage = $this->championshipMessageRepository->find($id);

        if (empty($championshipMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipMessages.singular')])
            );
        }

        $championshipMessage = $this->championshipMessageRepository->update($input, $id);

        return $this->sendResponse(
            new ChampionshipMessageResource($championshipMessage),
            __('messages.updated', ['model' => __('models/championshipMessages.singular')])
        );
    }

    /**
     * Remove the specified ChampionshipMessage from storage.
     * DELETE /championshipMessages/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var ChampionshipMessage $championshipMessage */
        $championshipMessage = $this->championshipMessageRepository->find($id);

        if (empty($championshipMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipMessages.singular')])
            );
        }

        $championshipMessage->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/championshipMessages.singular')])
        );
    }
}
