<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateChampionshipSubscriberAPIRequest;
use App\Http\Requests\API\UpdateChampionshipSubscriberAPIRequest;
use App\Models\Championship;
use App\Models\ChampionshipSubscriber;
use App\Models\Setting;
use App\Repositories\ChampionshipSubscriberRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\ChampionshipSubscriberResource;
use App\Models\User;
use Response;

/**
 * Class ChampionshipSubscriberController
 * @package App\Http\Controllers\API
 */

class ChampionshipSubscriberAPIController extends AppBaseController
{
    /** @var  ChampionshipSubscriberRepository */
    private $championshipSubscriberRepository;

    public function __construct(ChampionshipSubscriberRepository $championshipSubscriberRepo)
    {
        $this->championshipSubscriberRepository = $championshipSubscriberRepo;

        $this->middleware('subscribed')->only(['store']);
    }

    /**
     * Display a listing of the ChampionshipSubscriber.
     * GET|HEAD /championshipSubscribers
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $championshipSubscribers = $this->championshipSubscriberRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('championship_id', $request->championship_id);

        return $this->sendResponse(
            ChampionshipSubscriberResource::collection($championshipSubscribers),
            __('messages.retrieved', ['model' => __('models/championshipSubscribers.plural')])
        );
    }


    public function notify_championship_subscribers(Request $request): \Illuminate\Http\JsonResponse
    {
        $championshipSubscribers = $this->championshipSubscriberRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('championship_id', $request->championship_id);


        $championship = Championship::find($request->championship_id);

        $notification = [];
        $notification['title'] = $championship->title;
        $notification['content'] = $request->content;
        $notification['url'] = $request->championship_id;
        foreach ($championshipSubscribers as $subscriber) {
            $subscriber->user->notify(new \App\Notifications\Send($notification));
            sendFirebaseNotification($subscriber->user->token, $championship->title, $championship->content);
        }



        return $this->sendResponse(
            ChampionshipSubscriberResource::collection($championshipSubscribers),
            'Notification sent successfully'
        );
    }



    /**
     * Store a newly created ChampionshipSubscriber in storage.
     * POST /championshipSubscribers
     *
     * @param CreateChampionshipSubscriberAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateChampionshipSubscriberAPIRequest $request)
    {
        $championship = Championship::find($request->championship_id);
        $user = User::find(auth('sanctum')->id());

        if($championship->type=='special' && $user->special_trainer == 0){
            return $this->sendError('You are not a special user for this championship', 401);
        }
        $input = $request->all();
        $input['user_id'] = auth('sanctum')->id();
        $input['active'] = 1;
        $input['age_category_id'] = auth('sanctum')->user()->age_category_id;

        $championshipSubscriber = $this->championshipSubscriberRepository->create($input);

        if($championshipSubscriber->user_id){

            $user->notify(new \App\Notifications\ChampionshipActivate($championship));

            if($user->token){
                sendFirebaseNotification($user->token,__('messages.championship_activate'),$championship->title);
            }

        }

        return $this->sendResponse(
            new ChampionshipSubscriberResource($championshipSubscriber),
            __('messages.saved', ['model' => __('models/championshipSubscribers.singular')])
        );
    }


    public function add_championship_result_to_subscriber(UpdateChampionshipSubscriberAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        $championSubscriber = ChampionshipSubscriber::where('championship_id', $request->championship_id)->where('user_id', auth('sanctum')->id())->first();

        if (empty($championSubscriber)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipSubscribers.singular')])
            );
        }

        if ($request->correct_answers_count) {
            $input['student_points'] = $request->correct_answers_count * Setting::first()->championship_points;
        }

        if ($request->answering_period && $request->correct_answers_count){
            $championship = Championship::find($request->championship_id);
            $input['supposed_solved'] = exam_equation($championship->training_period * 60,$request->answering_period * 60,$request->correct_answers_count) ;
        }

        $championSubscriber = $this->championshipSubscriberRepository->update($input, $championSubscriber->id);

        return $this->sendResponse(
            new ChampionshipSubscriberResource($championSubscriber),
            __('messages.updated', ['model' => __('models/championshipSubscribers.singular')])
        );
    }

    public function collectStudentPoints(Request $request): \Illuminate\Http\JsonResponse
    {
        $championSubscriber = ChampionshipSubscriber::where('championship_id', $request->championship_id)->where('user_id', auth('sanctum')->id())->first();
        if ($championSubscriber->student_points_is_collected == 0) {
            $championSubscriber->student_points_is_collected = 1;
            $championSubscriber->save();

            $user = auth('sanctum')->user();
            $user->student_points = $user->student_points + $championSubscriber->student_points;
            $user->save();
        }
        return $this->sendResponse(
            new ChampionshipSubscriberResource($championSubscriber),
            __('messages.updated', ['model' => __('models/championshipSubscribers.singular')])
        );
    }

    /**
     * Display the specified ChampionshipSubscriber.
     * GET|HEAD /championshipSubscribers/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var ChampionshipSubscriber $championshipSubscriber */
        $championshipSubscriber = $this->championshipSubscriberRepository->find($id);

        if (empty($championshipSubscriber)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipSubscribers.singular')])
            );
        }

        return $this->sendResponse(
            new ChampionshipSubscriberResource($championshipSubscriber),
            __('messages.retrieved', ['model' => __('models/championshipSubscribers.singular')])
        );
    }

    /**
     * Update the specified ChampionshipSubscriber in storage.
     * PUT/PATCH /championshipSubscribers/{id}
     *
     * @param int $id
     * @param UpdateChampionshipSubscriberAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateChampionshipSubscriberAPIRequest $request)
    {
        $input = $request->all();

        /** @var ChampionshipSubscriber $championshipSubscriber */
        $championshipSubscriber = $this->championshipSubscriberRepository->find($id);

        if (empty($championshipSubscriber)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipSubscribers.singular')])
            );
        }

        $championshipSubscriber = $this->championshipSubscriberRepository->update($input, $id);

        return $this->sendResponse(
            new ChampionshipSubscriberResource($championshipSubscriber),
            __('messages.updated', ['model' => __('models/championshipSubscribers.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $championshipSubscriber = $this->championshipSubscriberRepository->find($id);
        $championshipSubscriber->active = !$championshipSubscriber->active;
        $championshipSubscriber->save();

        if($championshipSubscriber->user_id){
            $championship = Championship::find($championshipSubscriber->championship_id);
            $user = User::find($championshipSubscriber->user_id);//
            $user->notify(new \App\Notifications\ChampionshipActivate($championship));
            
            if($user->token){
                sendFirebaseNotification($user->token,__('messages.championship_activate'),$championship->title);
            }

        }

        return $this->sendResponse(
            new ChampionshipSubscriberResource($championshipSubscriber),
            __('messages.updated', ['model' => __('models/championshipSubscribers.singular')])
        );
    }

    /**
     * Remove the specified ChampionshipSubscriber from storage.
     * DELETE /championshipSubscribers/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var ChampionshipSubscriber $championshipSubscriber */
        $championshipSubscriber = $this->championshipSubscriberRepository->find($id);

        if (empty($championshipSubscriber)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/championshipSubscribers.singular')])
            );
        }

        $championshipSubscriber->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/championshipSubscribers.singular')])
        );
    }
}
