<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateCompetitionAPIRequest;
use App\Http\Requests\API\UpdateCompetitionAPIRequest;
use App\Http\Resources\QuestionResource;
use App\Http\Resources\StudentResource;
use App\Http\Resources\TrainingQuestionResource;
use App\Http\Resources\TrainingResource;
use App\Models\Competition;
use App\Models\CompetitionSubscriber;
use App\Models\Models\CompetitionQuestion;
use App\Models\Models\SubscriberCompetitionQuestionAnswer;
use App\Models\Question;
use App\Models\User;
use App\Repositories\CompetitionRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\CompetitionResource;
use Carbon\Carbon;
use Response;
use Validator;
/**
 * Class CompetitionController
 * @package App\Http\Controllers\API
 */

class CompetitionAPIController extends AppBaseController
{
    /** @var  CompetitionRepository */
    private $competitionRepository;

    public function __construct(CompetitionRepository $competitionRepo)
    {
        $this->competitionRepository = $competitionRepo;

        $this->middleware('subscribed')->only(['store']);
    }

    /**
     * Display a listing of the Competition.
     * GET|HEAD /competitions
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request): \Illuminate\Http\JsonResponse
    {
        $competitions = $this->competitionRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        // if ($request->user()?->role =='student'){
        //     $competitions = $competitions->where('user_id',auth('sanctum')->id())->orWhereHas('subscribers', fn ($query) => $query->where('active',1)->where('user_id', auth('sanctum')->id()));
        // }

        if ($request->user()?->role =='trainer'){
            $competitions = $competitions->where('user_id',auth('sanctum')->id());
        }

        if ($request->get('trainer_id')){
            $competitions = $competitions->where('user_id',$request->trainer_id);
        }

        if ($request->get('student_id')){
            $competitions = $competitions->where('user_id',$request->student_id);
        }

        return $this->sendResponse(
            CompetitionResource::collection($competitions),
            __('messages.retrieved', ['model' => __('models/competitions.plural')])
        );
    }

    public function my_competitions(Request $request): \Illuminate\Http\JsonResponse
    {
            $competitions = Competition::where('active',1)->where('user_id',auth('sanctum')->id())->orWhereHas('subscribers', fn ($query) => $query->where('active',1)->where('user_id', auth('sanctum')->id()))->orderBy('created_at', 'desc')->get();

            return $this->sendResponse(
                CompetitionResource::collection($competitions),
                __('messages.retrieved', ['model' => __('models/competitions.plural')])
            );
    }

    /**
     * Store a newly created Competition in storage.
     * POST /competitions
     *
     * @param CreateCompetitionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateCompetitionAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        $input['user_id'] = auth('sanctum')->id();

        if(isset($request->competition_datetime)){
            $input['expired_at'] = Carbon::parse($request->competition_datetime)->addMinutes($request->training_period)->timezone('Africa/Cairo');
        }


        $competition = $this->competitionRepository->create($input);

        $data = [];

        $all_questions = $this->competitionRepository->get_questions($competition);

       if(is_countable($all_questions) == false){
            return  $all_questions ;
        }

        $competition->questions()->sync($all_questions->pluck('id')->toArray());

        if ($competition->user?->role =='student'){
            $competition_subscriber= new CompetitionSubscriber();
            $competition_subscriber->user_id =  $competition->user_id;
            $competition_subscriber->competition_id = $competition->id;
            $competition_subscriber->save();
        }

        //$competition->subscribers()->create($request->subscribers);
        foreach ($request->subscribers as $subscriber_id){
            $competition_subscriber= new CompetitionSubscriber();
            $competition_subscriber->user_id = $subscriber_id;
            $competition_subscriber->competition_id = $competition->id;
            $competition_subscriber->active = 0;
            $competition_subscriber->save();

            $user = User::find($subscriber_id);//
            $user->notify(new \App\Notifications\Competition($competition));

            sendFirebaseNotification($user->token,trans('messages.competition_invitation'),$competition->user?->name);
        }

        if (count($competition->subscribers) ==0){
            $competition->delete();
            return $this->sendError(trans('messages.Competition_not_shared_to_any_subscribers'));
        }



        $data['competition'] =  new CompetitionResource($competition);
        $data['questions'] = QuestionResource::collection($all_questions);

        return $this->sendResponse(
            $data,
            __('messages.saved', ['model' => __('models/competitions.singular')])
        );
    }


    public function share_competition_for_subscribers($id, Request $request): \Illuminate\Http\JsonResponse
    {
        $competition = $this->competitionRepository->find($id);

        foreach ($request->subscribers as $subscriber_id){
            $competition_subscriber= new CompetitionSubscriber();
            $competition_subscriber->user_id = $subscriber_id;
            $competition_subscriber->competition_id = $competition->id;
            $competition_subscriber->save();


            $user = User::find($subscriber_id);
            if($user){
            $user->notify(new \App\Notifications\Competition($competition));

            sendFirebaseNotification($user->token,trans('messages.competition_invitation'),$competition->user?->name);
            }
        }



        return $this->sendResponse(
            new CompetitionResource($competition),
            __('messages.saved', ['model' => __('models/competitions.singular')])
        );
    }

    /**
     * Display the specified Competition.
     * GET|HEAD /competitions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id): \Illuminate\Http\JsonResponse
    {
        /** @var Competition $competition */
        $competition = $this->competitionRepository->find($id);

        if (empty($competition)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitions.singular')])
            );
        }

        $data['competition'] =  new CompetitionResource($competition);
        $data['questions'] = QuestionResource::collection($competition->questions);

        return $this->sendResponse(
            $data,
            __('messages.saved', ['model' => __('models/competitions.singular')])
        );
    }

    /**
     * Update the specified Competition in storage.
     * PUT/PATCH /competitions/{id}
     *
     * @param int $id
     * @param UpdateCompetitionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateCompetitionAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        /** @var Competition $competition */
        $competition = $this->competitionRepository->find($id);

        if (empty($competition)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitions.singular')])
            );
        }

        $competition = $this->competitionRepository->update($input, $id);

        return $this->sendResponse(
            new CompetitionResource($competition),
            __('messages.updated', ['model' => __('models/competitions.singular')])
        );
    }


    public function answer_question_competition()
    {
        if(SubscriberCompetitionQuestionAnswer::where('question_id',\request('question_id'))->first() == null){
            $answer_question = new SubscriberCompetitionQuestionAnswer();
            $answer_question->competition_id = \request('competition_id');
            $answer_question->question_id = \request('question_id');
            $answer_question->user_id = auth('sanctum')->id();
            $answer_question->is_correct = \request('is_correct');
            $answer_question->wrong_answer = \request('wrong_answer');
            $answer_question->save();
        }

        return $this->sendResponse(
            new TrainingQuestionResource($answer_question),
            __('messages.updated', ['model' => __('models/competitions.singular')])
        );
    }

    public function answer_competition_questions()
    {
        $validator = Validator::make(\request()->all(), [
            'questions' => 'required|array',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first());
        }

        $competition = $this->competitionRepository->find(\request('competition_id'));

        if (empty($competition)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitions.singular')])
            );
        }

        foreach (\request('questions') as $i=> $question_id){
            if(SubscriberCompetitionQuestionAnswer::where('competition_id',\request('competition_id'))->where('user_id',auth('sanctum')->id())->where('question_id',$question_id)->first() == null){
                $answer_question = new SubscriberCompetitionQuestionAnswer();
                $answer_question->competition_subscriber_id = CompetitionSubscriber::where('competition_id',\request('competition_id'))->where('user_id',auth('sanctum')->id())->first()->id;
                $answer_question->competition_id = \request('competition_id');
                $answer_question->question_id = $question_id;
                $answer_question->user_id = auth('sanctum')->id();
                $answer_question->is_correct = \request('is_correct')[$i];
                $answer_question->wrong_answer = \request('wrong_answer')[$i];
                $answer_question->save();
            }
        }


        return $this->sendResponse(
            new CompetitionResource($competition),
            __('messages.updated', ['model' => __('models/competitions.singular')])
        );
    }


    public function competition_questions_answers(): \Illuminate\Http\JsonResponse
    {
        $questions_answers = SubscriberCompetitionQuestionAnswer::where('competition_id',\request('competition_id'))->where('user_id',\request('user_id'))->get();

        return $this->sendResponse(
            TrainingQuestionResource::collection($questions_answers),
            __('messages.updated', ['model' => __('models/competitions.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $competition = $this->competitionRepository->find($id);
        $competition->active = !$competition->active;
        $competition->save();

        return $this->sendResponse(
            new CompetitionResource($competition),
            __('messages.updated', ['model' => __('models/competitions.singular')])
        );
    }

    /**
     * Remove the specified Competition from storage.
     * DELETE /competitions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id): \Illuminate\Http\JsonResponse
    {
        /** @var Competition $competition */
        $competition = $this->competitionRepository->find($id);

        if (empty($competition)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitions.singular')])
            );
        }

        $competition->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/competitions.singular')])
        );
    }
}
