<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateCompetitionMessageAPIRequest;
use App\Http\Requests\API\UpdateCompetitionMessageAPIRequest;
use App\Models\CompetitionMessage;
use App\Repositories\CompetitionMessageRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\CompetitionMessageResource;
use Response;

/**
 * Class CompetitionMessageController
 * @package App\Http\Controllers\API
 */

class CompetitionMessageAPIController extends AppBaseController
{
    /** @var  CompetitionMessageRepository */
    private $competitionMessageRepository;

    public function __construct(CompetitionMessageRepository $competitionMessageRepo)
    {
        $this->competitionMessageRepository = $competitionMessageRepo;
    }

    /**
     * Display a listing of the CompetitionMessage.
     * GET|HEAD /competitionMessages
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request): \Illuminate\Http\JsonResponse
    {
        $competitionMessages = $this->competitionMessageRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('competition_id',$request->competition_id);

        return $this->sendResponse(
            CompetitionMessageResource::collection($competitionMessages),
            __('messages.retrieved', ['model' => __('models/competitionMessages.plural')])
        );
    }

    /**
     * Store a newly created CompetitionMessage in storage.
     * POST /competitionMessages
     *
     * @param CreateCompetitionMessageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateCompetitionMessageAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        $input['user_id'] = auth('sanctum')->id();

        if($request->file('file')){
            $input['file'] = $this->saveImage($request->file('file'), 'competitionMessages');
        }

        $competitionMessage = $this->competitionMessageRepository->create($input);

        event(new \App\Events\CompetitionMessage( new CompetitionMessageResource($competitionMessage)));

        return $this->sendResponse(
            new CompetitionMessageResource($competitionMessage),
            __('messages.saved', ['model' => __('models/competitionMessages.singular')])
        );
    }

    /**
     * Display the specified CompetitionMessage.
     * GET|HEAD /competitionMessages/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id): \Illuminate\Http\JsonResponse
    {
        /** @var CompetitionMessage $competitionMessage */
        $competitionMessage = $this->competitionMessageRepository->find($id);

        if (empty($competitionMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitionMessages.singular')])
            );
        }

        return $this->sendResponse(
            new CompetitionMessageResource($competitionMessage),
            __('messages.retrieved', ['model' => __('models/competitionMessages.singular')])
        );
    }

    /**
     * Update the specified CompetitionMessage in storage.
     * PUT/PATCH /competitionMessages/{id}
     *
     * @param int $id
     * @param UpdateCompetitionMessageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateCompetitionMessageAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        /** @var CompetitionMessage $competitionMessage */
        $competitionMessage = $this->competitionMessageRepository->find($id);

        if (empty($competitionMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitionMessages.singular')])
            );
        }

        $competitionMessage = $this->competitionMessageRepository->update($input, $id);

        return $this->sendResponse(
            new CompetitionMessageResource($competitionMessage),
            __('messages.updated', ['model' => __('models/competitionMessages.singular')])
        );
    }

    /**
     * Remove the specified CompetitionMessage from storage.
     * DELETE /competitionMessages/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id): \Illuminate\Http\JsonResponse
    {
        /** @var CompetitionMessage $competitionMessage */
        $competitionMessage = $this->competitionMessageRepository->find($id);

        if (empty($competitionMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/competitionMessages.singular')])
            );
        }

        $competitionMessage->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/competitionMessages.singular')])
        );
    }
}
