<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\AppBaseController;
use App\Http\Requests\API\CreateContactAPIRequest;
use App\Http\Requests\API\UpdateContactAPIRequest;
use App\Models\Contact;
use App\Models\User;
use App\Repositories\ContactRepository;
use Illuminate\Http\Request;
use App\Http\Resources\ContactResource;
use App\Mail\ContactReply;
use Response;
use Mail;
/**
 * Class ContactController
 * @package App\Http\Controllers\API
 */

class ContactAPIController extends AppBaseController
{
    /** @var  ContactRepository */
    private $contactRepository;

    public function __construct(ContactRepository $contactRepo)
    {
        $this->contactRepository = $contactRepo;
    }

    /**
     * Display a listing of the Contact.
     * GET|HEAD /contacts
     *
     * @param Request $request
     * @return Response
     */
    public function index(Request $request)
    {
        $contacts = $this->contactRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            ContactResource::collection($contacts),
            __('messages.retrieved', ['model' => __('models/contacts.plural')])
        );
    }

    /**
     * Store a newly created Contact in storage.
     * POST /contacts
     *
     * @param CreateContactAPIRequest $request
     *
     * @return Response
     */
    public function store(CreateContactAPIRequest $request)
    {
        $input = $request->all();
        if (auth('sanctum')->check()) {
         $input['type'] = auth('sanctum')->user()->role;
         $input['user_id'] = auth('sanctum')->id();
        }
        $contact = $this->contactRepository->create($input);

        $admin = User::where('role','admin')->first();
        $admin->notify(new \App\Notifications\Contact($contact));

        return $this->sendResponse(
            new ContactResource($contact),
            __('messages.saved', ['model' => __('models/contacts.singular')])
        );
    }

    /**
     * Display the specified Contact.
     * GET|HEAD /contacts/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var Contact $contact */
        $contact = $this->contactRepository->find($id);

        if (empty($contact)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/contacts.singular')])
            );
        }

        return $this->sendResponse(
            new ContactResource($contact),
            __('messages.retrieved', ['model' => __('models/contacts.singular')])
        );
    }

    /**
     * Update the specified Contact in storage.
     * PUT/PATCH /contacts/{id}
     *
     * @param int $id
     * @param UpdateContactAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateContactAPIRequest $request)
    {
        $input = $request->all();

        /** @var Contact $contact */
        $contact = $this->contactRepository->find($id);

        if (empty($contact)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/contacts.singular')])
            );
        }


        Mail::to($contact->email)->send(new ContactReply($request->reply));

        $this->contactRepository->update($input, $id);

        if($contact->user_id){
            $user = User::find($contact->user_id);//
            $user->notify(new \App\Notifications\Contact($contact));

            if($user->token){
                sendFirebaseNotification($user->token,__('models/contacts.reply'),$contact->reply);
            }

        }


        return $this->sendResponse(
            new ContactResource($contact),
            __('messages.updated', ['model' => __('models/contacts.singular')])
        );
    }

    /**
     * Remove the specified Contact from storage.
     * DELETE /contacts/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var Contact $contact */
        $contact = $this->contactRepository->find($id);

        if (empty($contact)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/contacts.singular')])
            );
        }

        $contact->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/contacts.singular')])
        );
    }
}
