<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateCouponAPIRequest;
use App\Http\Requests\API\UpdateCouponAPIRequest;
use App\Http\Resources\CouponDiscountResource;
use App\Models\Coupon;
use App\Repositories\CouponRepository;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\CouponResource;
use Response;

/**
 * Class CouponController
 * @package App\Http\Controllers\API
 */

class CouponAPIController extends AppBaseController
{
    /** @var  CouponRepository */
    private $couponRepository;

    public function __construct(CouponRepository $couponRepo)
    {
        $this->couponRepository = $couponRepo;
    }

    /**
     * Display a listing of the Coupon.
     * GET|HEAD /coupons
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $coupons = $this->couponRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            CouponResource::collection($coupons),
            __('messages.retrieved', ['model' => __('models/coupons.plural')])
        );
    }

    /**
     * Store a newly created Coupon in storage.
     * POST /coupons
     *
     * @param CreateCouponAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateCouponAPIRequest $request)
    {
        $input = $request->all();

        $coupon = $this->couponRepository->create($input);

        return $this->sendResponse(
            new CouponResource($coupon),
            __('messages.saved', ['model' => __('models/coupons.singular')])
        );
    }


    public function apply_coupon(): \Illuminate\Http\JsonResponse
    {
        /** @var Coupon $coupon */
        $coupon = Coupon::where('code',request('code'))->first();;

        if (empty($coupon)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/coupons.singular')])
            );
        }

        if ($coupon->end_date <> null){
            $current_date = Carbon::now()->format('Y-m-d');
            if ($coupon->end_date < $current_date ) {
                return $this->sendError(
                    __('messages.expired', ['model' => __('models/coupons.singular')])
                );
            }
        }


        if ($coupon->active == 0) {
            return $this->sendError(
                __('messages.not_active', ['model' => __('models/coupons.singular')])
            );
        }

        if ($coupon->max_use <> null && $coupon->use_count >= $coupon->max_use) {
            return $this->sendError(
                __('messages.max', ['model' => __('models/coupons.singular')])
            );
        }

        return $this->sendResponse(
            new CouponDiscountResource($coupon),
            __('messages.retrieved', ['model' => __('models/coupons.singular')])
        );
    }

    /**
     * Display the specified Coupon.
     * GET|HEAD /coupons/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Coupon $coupon */
        $coupon = Coupon::where('code',$id)->first();;

        if (empty($coupon)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/coupons.singular')])
            );
        }

        $current_date = Carbon::now()->format('Y-m-d');
        if ($coupon->start_date <= $current_date || $coupon->end_date > $current_date ) {
            return $this->sendError(
                __('messages.expired', ['model' => __('models/coupons.singular')])
            );
        }

        if ($coupon->is_active == 0) {
            return $this->sendError(
                __('messages.not_active', ['model' => __('models/coupons.singular')])
            );
        }

        return $this->sendResponse(
            new CouponDiscountResource($coupon),
            __('messages.retrieved', ['model' => __('models/coupons.singular')])
        );
    }

    /**
     * Update the specified Coupon in storage.
     * PUT/PATCH /coupons/{id}
     *
     * @param int $id
     * @param UpdateCouponAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateCouponAPIRequest $request)
    {
        $input = $request->all();

        /** @var Coupon $coupon */
        $coupon = $this->couponRepository->find($id);

        if (empty($coupon)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/coupons.singular')])
            );
        }

        $coupon = $this->couponRepository->update($input, $id);

        return $this->sendResponse(
            new CouponResource($coupon),
            __('messages.updated', ['model' => __('models/coupons.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $coupon = $this->couponRepository->find($id);
        $coupon->active = !$coupon->active;
        $coupon->save();

        return $this->sendResponse(
            new CouponResource($coupon),
            __('messages.updated', ['model' => __('models/coupons.singular')])
        );
    }

    /**
     * Remove the specified Coupon from storage.
     * DELETE /coupons/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Coupon $coupon */
        $coupon = $this->couponRepository->find($id);

        if (empty($coupon)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/coupons.singular')])
            );
        }

        $coupon->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/coupons.singular')])
        );
    }
}
