<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateIntroPageAPIRequest;
use App\Http\Requests\API\UpdateIntroPageAPIRequest;
use App\Models\IntroPage;
use App\Repositories\IntroPageRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\IntroPageResource;
use Response;

/**
 * Class IntroPageController
 * @package App\Http\Controllers\API
 */

class IntroPageAPIController extends AppBaseController
{
    /** @var  IntroPageRepository */
    private $introPageRepository;

    public function __construct(IntroPageRepository $introPageRepo)
    {
        $this->introPageRepository = $introPageRepo;
    }

    /**
     * Display a listing of the IntroPage.
     * GET|HEAD /introPages
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $introPages = $this->introPageRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('type',$request->get('type'));

        return $this->sendResponse(
            IntroPageResource::collection($introPages),
            __('messages.retrieved', ['model' => __('models/introPages.plural')])
        );
    }

    /**
     * Store a newly created IntroPage in storage.
     * POST /introPages
     *
     * @param CreateIntroPageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateIntroPageAPIRequest $request)
    {
        $input = $request->all();

        $input['image'] =  $this->saveImage($request->file('image'), 'introPages');

        $introPage = $this->introPageRepository->create($input);

        return $this->sendResponse(
            new IntroPageResource($introPage),
            __('messages.saved', ['model' => __('models/introPages.singular')])
        );
    }

    /**
     * Display the specified IntroPage.
     * GET|HEAD /introPages/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var IntroPage $introPage */
        $introPage = $this->introPageRepository->find($id);

        if (empty($introPage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/introPages.singular')])
            );
        }

        return $this->sendResponse(
            new IntroPageResource($introPage),
            __('messages.retrieved', ['model' => __('models/introPages.singular')])
        );
    }

    /**
     * Update the specified IntroPage in storage.
     * PUT/PATCH /introPages/{id}
     *
     * @param int $id
     * @param UpdateIntroPageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateIntroPageAPIRequest $request)
    {
        $input = $request->all();

        /** @var IntroPage $introPage */
        $introPage = $this->introPageRepository->find($id);

        if (empty($introPage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/introPages.singular')])
            );
        }

        $input['image'] =  $request->hasFile('image') ?$this->updateImage($introPage->image,$request->file('image'), 'introPages'):$introPage->image;

        $introPage = $this->introPageRepository->update($input, $id);

        return $this->sendResponse(
            new IntroPageResource($introPage),
            __('messages.updated', ['model' => __('models/introPages.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $introPage = $this->introPageRepository->find($id);
        $introPage->active = !$introPage->active;
        $introPage->save();

        return $this->sendResponse(
            new IntroPageResource($introPage),
            __('messages.updated', ['model' => __('models/introPages.singular')])
        );
    }

    /**
     * Remove the specified IntroPage from storage.
     * DELETE /introPages/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var IntroPage $introPage */
        $introPage = $this->introPageRepository->find($id);

        if (empty($introPage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/introPages.singular')])
            );
        }

        $introPage->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/introPages.singular')])
        );
    }
}
