<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateLevelCategoryAPIRequest;
use App\Http\Requests\API\UpdateLevelCategoryAPIRequest;
use App\Models\LevelCategory;
use App\Repositories\LevelCategoryRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\LevelCategoryResource;
use Response;

/**
 * Class LevelCategoryController
 * @package App\Http\Controllers\API
 */

class LevelCategoryAPIController extends AppBaseController
{
    /** @var  LevelCategoryRepository */
    private $levelCategoryRepository;

    public function __construct(LevelCategoryRepository $levelCategoryRepo)
    {
        $this->levelCategoryRepository = $levelCategoryRepo;
    }

    /**
     * Display a listing of the LevelCategory.
     * GET|HEAD /levelCategories
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $levelCategories = $this->levelCategoryRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if (\request('type_level_id')){
            $levelCategories->where('type_level_id',\request('type_level_id'));
        }

        return $this->sendResponse(
            LevelCategoryResource::collection($levelCategories),
            __('messages.retrieved', ['model' => __('models/levelCategories.plural')])
        );
    }

    /**
     * Store a newly created LevelCategory in storage.
     * POST /levelCategories
     *
     * @param CreateLevelCategoryAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateLevelCategoryAPIRequest $request)
    {
        $levelCategory = LevelCategory::where('title','LIKE',json_encode($request->title,JSON_UNESCAPED_UNICODE ))->where('type_level_id',$request->type_level_id)->first();
        if ($levelCategory == null) {
            $input = $request->all();

            if ($request->file('icon')){
                $input['icon'] =  $this->saveImage($request->file('icon'), 'levelCategories');
            }


            $levelCategory = $this->levelCategoryRepository->create($input);
        }

        return $this->sendResponse(
            new LevelCategoryResource($levelCategory),
            __('messages.saved', ['model' => __('models/levelCategories.singular')])
        );
    }

    /**
     * Display the specified LevelCategory.
     * GET|HEAD /levelCategories/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var LevelCategory $levelCategory */
        $levelCategory = $this->levelCategoryRepository->find($id);

        if (empty($levelCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/levelCategories.singular')])
            );
        }

        return $this->sendResponse(
            new LevelCategoryResource($levelCategory),
            __('messages.retrieved', ['model' => __('models/levelCategories.singular')])
        );
    }

    /**
     * Update the specified LevelCategory in storage.
     * PUT/PATCH /levelCategories/{id}
     *
     * @param int $id
     * @param UpdateLevelCategoryAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateLevelCategoryAPIRequest $request)
    {
        $input = $request->all();

        /** @var LevelCategory $levelCategory */
        $levelCategory = $this->levelCategoryRepository->find($id);

        if (empty($levelCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/levelCategories.singular')])
            );
        }

        $input['icon'] =  $request->hasFile('icon') ?$this->updateImage($levelCategory->icon,$request->file('icon'), 'levelCategories'):$levelCategory->icon;

        $levelCategory = $this->levelCategoryRepository->update($input, $id);

        return $this->sendResponse(
            new LevelCategoryResource($levelCategory),
            __('messages.updated', ['model' => __('models/levelCategories.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $levelCategory = $this->levelCategoryRepository->find($id);
        $levelCategory->active = !$levelCategory->active;
        $levelCategory->save();

        return $this->sendResponse(
            new LevelCategoryResource($levelCategory),
            __('messages.updated', ['model' => __('models/levelCategories.singular')])
        );
    }

    /**
     * Remove the specified LevelCategory from storage.
     * DELETE /levelCategories/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var LevelCategory $levelCategory */
        $levelCategory = $this->levelCategoryRepository->find($id);

        if (empty($levelCategory)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/levelCategories.singular')])
            );
        }

        $levelCategory->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/levelCategories.singular')])
        );
    }
}
