<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreatePackageAPIRequest;
use App\Http\Requests\API\UpdatePackageAPIRequest;
use App\Http\Resources\SubscriptionResource;
use App\Models\Coupon;
use App\Models\Package;
use App\Models\Subscription;
use App\Repositories\PackageRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\PackageResource;
use Response;
use Carbon\Carbon;

/**
 * Class PackageController
 * @package App\Http\Controllers\API
 */

class PackageAPIController extends AppBaseController
{
    /** @var  PackageRepository */
    private $packageRepository;

    public function __construct(PackageRepository $packageRepo)
    {
        $this->packageRepository = $packageRepo;
    }

    /**
     * Display a listing of the Package.
     * GET|HEAD /packages
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $packages = $this->packageRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('type',$request->get('type'));

        if ($request->user()->role ==  $request->get('type')) {
            $packages = $packages->where('active',1);
        }

        return $this->sendResponse(
            PackageResource::collection($packages),
            __('messages.retrieved', ['model' => __('models/packages.plural')])
        );
    }

    public function subscribe_package(): \Illuminate\Http\JsonResponse
    {

        $package = $this->packageRepository->find(\request('package_id'));

        if($package->price == 0){
            if (auth('sanctum')->user()->free_trial == 1) {
                return $this->sendError(
                    __('messages.free_trial', ['model' => __('models/packages.singular')])
                );
            }
        }

        if (empty($package)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/packages.singular')])
            );
        }


        if(request('coupon')){
            $coupon = Coupon::where('code', request('coupon'))->where('active',1)
                ->first();

                if (empty($coupon)) {
                    return $this->sendError(
                        __('messages.not_found', ['model' => __('models/coupons.singular')])
                    );
                }

                if ($coupon->end_date <> null){
                    $current_date = Carbon::now()->format('Y-m-d');
                    if ($coupon->end_date < $current_date ) {
                        return $this->sendError(
                            __('messages.expired', ['model' => __('models/coupons.singular')])
                        );
                    }
                }


                if ($coupon->active == 0) {
                    return $this->sendError(
                        __('messages.not_active', ['model' => __('models/coupons.singular')])
                    );
                }

                if ($coupon->max_use <> null && $coupon->use_count >= $coupon->max_use) {
                    return $this->sendError(
                        __('messages.max', ['model' => __('models/coupons.singular')])
                    );
                }


            if($coupon){
                if($coupon->max_use == null || $coupon->use_count == null || $coupon->max_use > $coupon->use_count){
                    $discount_coupon = ($coupon->discount/100) * $package->price;
                    $coupon->use_count += 1;
                    $coupon->save();
                }
            }
        }

        Subscription::where('user_id',auth('sanctum')->id())->delete();

        $subscription = new Subscription();
        $subscription->user_id = auth('sanctum')->id();
        $subscription->package_id = $package->id;
        $subscription->start_date = Carbon::now()->timezone('Africa/Cairo')->format('Y-m-d H:i:s');
        $subscription->end_date = $package->duration_type == 'month' ? Carbon::now()->addMonths($package->duration)->timezone('Africa/Cairo')->format('Y-m-d H:i:s') : ($package->duration_type == 'day' ? ($package->duration > 1 ? Carbon::now()->addDays($package->duration)->timezone('Africa/Cairo')->format('Y-m-d H:i:s') :  Carbon::now()->addHours(24)->timezone('Africa/Cairo')->format('Y-m-d H:i:s')) : Carbon::now()->addYears($package->duration)->timezone('Africa/Cairo')->format('Y-m-d H:i:s'));
        $subscription->active = true;
        $subscription->coupon_id = isset($coupon)&& isset($discount_coupon) ?$coupon->id:null;
        $subscription->cost = isset($discount_coupon)?round($package->price - $discount_coupon, 2):$package->price;
        $subscription->save();

        if($package->price == 0){
          $user = auth('sanctum')->user();
          $user->free_trial = 1;
          $user->save();
        }


        return $this->sendResponse(
            new SubscriptionResource($subscription),
            __('messages.saved', ['model' => __('models/subscriptions.singular')])
        );
    }
    public function my_package(): \Illuminate\Http\JsonResponse
    {
        $subscription = Subscription::where('user_id',auth('sanctum')->id())->whereDate('end_date','>',now())->orderBy('created_at', 'desc')->first();

        if (empty($subscription)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/subscriptions.singular')])
            );
        }


        return $this->show($subscription->package_id);
    }

    /**
     * Store a newly created Package in storage.
     * POST /packages
     *
     * @param CreatePackageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreatePackageAPIRequest $request)
    {
        $input = $request->all();

        $package = $this->packageRepository->create($input);

        return $this->sendResponse(
            new PackageResource($package),
            __('messages.saved', ['model' => __('models/packages.singular')])
        );
    }

    /**
     * Display the specified Package.
     * GET|HEAD /packages/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Package $package */
        $package = $this->packageRepository->find($id);

        if (empty($package)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/packages.singular')])
            );
        }

        return $this->sendResponse(
            new PackageResource($package),
            __('messages.retrieved', ['model' => __('models/packages.singular')])
        );
    }

    /**
     * Update the specified Package in storage.
     * PUT/PATCH /packages/{id}
     *
     * @param int $id
     * @param UpdatePackageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdatePackageAPIRequest $request): \Illuminate\Http\JsonResponse
    {
        $input = $request->all();

        /** @var Package $package */
        $package = $this->packageRepository->find($id);

        if (empty($package)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/packages.singular')])
            );
        }

        $package = $this->packageRepository->update($input, $id);

        return $this->sendResponse(
            new PackageResource($package),
            __('messages.updated', ['model' => __('models/packages.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $package = $this->packageRepository->find($id);
        $package->active = !$package->active;
        $package->save();

        return $this->sendResponse(
            new PackageResource($package),
            __('messages.updated', ['model' => __('models/packages.singular')])
        );
    }

    /**
     * Remove the specified Package from storage.
     * DELETE /packages/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var Package $package */
        $package = $this->packageRepository->find($id);

        if (empty($package)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/packages.singular')])
            );
        }

        $package->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/packages.singular')])
        );
    }
}
