<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreatePointLevelAPIRequest;
use App\Http\Requests\API\UpdatePointLevelAPIRequest;
use App\Models\PointLevel;
use App\Repositories\PointLevelRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\PointLevelResource;
use Response;

/**
 * Class PointLevelController
 * @package App\Http\Controllers\API
 */

class PointLevelAPIController extends AppBaseController
{
    /** @var  PointLevelRepository */
    private $pointLevelRepository;

    public function __construct(PointLevelRepository $pointLevelRepo)
    {
        $this->pointLevelRepository = $pointLevelRepo;
    }

    /**
     * Display a listing of the PointLevel.
     * GET|HEAD /pointLevels
     *
     * @param Request $request
     * @return Response
     */
    public function index(Request $request)
    {
        $pointLevels = $this->pointLevelRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            PointLevelResource::collection($pointLevels),
            __('messages.retrieved', ['model' => __('models/pointLevels.plural')])
        );
    }

    /**
     * Store a newly created PointLevel in storage.
     * POST /pointLevels
     *
     * @param CreatePointLevelAPIRequest $request
     *
     * @return Response
     */
    public function store(CreatePointLevelAPIRequest $request)
    {
        $input = $request->all();

        $pointLevel = $this->pointLevelRepository->create($input);

        return $this->sendResponse(
            new PointLevelResource($pointLevel),
            __('messages.saved', ['model' => __('models/pointLevels.singular')])
        );
    }

    /**
     * Display the specified PointLevel.
     * GET|HEAD /pointLevels/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var PointLevel $pointLevel */
        $pointLevel = $this->pointLevelRepository->find($id);

        if (empty($pointLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/pointLevels.singular')])
            );
        }

        return $this->sendResponse(
            new PointLevelResource($pointLevel),
            __('messages.retrieved', ['model' => __('models/pointLevels.singular')])
        );
    }

    /**
     * Update the specified PointLevel in storage.
     * PUT/PATCH /pointLevels/{id}
     *
     * @param int $id
     * @param UpdatePointLevelAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdatePointLevelAPIRequest $request)
    {
        $input = $request->all();

        /** @var PointLevel $pointLevel */
        $pointLevel = $this->pointLevelRepository->find($id);

        if (empty($pointLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/pointLevels.singular')])
            );
        }

        $pointLevel = $this->pointLevelRepository->update($input, $id);

        return $this->sendResponse(
            new PointLevelResource($pointLevel),
            __('messages.updated', ['model' => __('models/pointLevels.singular')])
        );
    }

    /**
     * Remove the specified PointLevel from storage.
     * DELETE /pointLevels/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var PointLevel $pointLevel */
        $pointLevel = $this->pointLevelRepository->find($id);

        if (empty($pointLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/pointLevels.singular')])
            );
        }

        $pointLevel->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/pointLevels.singular')])
        );
    }
}
