<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateQuestionAPIRequest;
use App\Http\Requests\API\UpdateQuestionAPIRequest;
use App\Models\Question;
use App\Repositories\QuestionRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\QuestionResource;
use Response;

/**
 * Class QuestionController
 * @package App\Http\Controllers\API
 */

class QuestionAPIController extends AppBaseController
{
    /** @var  QuestionRepository */
    private $questionRepository;

    public function __construct(QuestionRepository $questionRepo)
    {
        $this->questionRepository = $questionRepo;
    }

    /**
     * Display a listing of the Question.
     * GET|HEAD /questions
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $questions = $this->questionRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            QuestionResource::collection($questions),
            __('messages.retrieved', ['model' => __('models/questions.plural')])
        );
    }

    /**
     * Store a newly created Question in storage.
     * POST /questions
     *
     * @param CreateQuestionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateQuestionAPIRequest $request)
    {
        $input = $request->all();

        if ($request->numbers)
        $input['numbers'] = json_encode($request->numbers);

        if ($request->choices)
        $input['choices'] = json_encode($request->choices);


        $question = $this->questionRepository->create($input);

        return $this->sendResponse(
            new QuestionResource($question),
            __('messages.saved', ['model' => __('models/questions.singular')])
        );
    }

    /**
     * Display the specified Question.
     * GET|HEAD /questions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Question $question */
        $question = $this->questionRepository->find($id);

        if (empty($question)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questions.singular')])
            );
        }

        return $this->sendResponse(
            new QuestionResource($question),
            __('messages.retrieved', ['model' => __('models/questions.singular')])
        );
    }

    /**
     * Update the specified Question in storage.
     * PUT/PATCH /questions/{id}
     *
     * @param int $id
     * @param UpdateQuestionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateQuestionAPIRequest $request)
    {
        $input = $request->all();

        /** @var Question $question */
        $question = $this->questionRepository->find($id);

        if (empty($question)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questions.singular')])
            );
        }

        if ($request->numbers)
            $input['numbers'] = json_encode($request->numbers);

        if ($request->choices)
            $input['choices'] = json_encode($request->choices);

        $question = $this->questionRepository->update($input, $id);

        return $this->sendResponse(
            new QuestionResource($question),
            __('messages.updated', ['model' => __('models/questions.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $question = $this->questionRepository->find($id);
        $question->active = !$question->active;
        $question->save();

        return $this->sendResponse(
            new QuestionResource($question),
            __('messages.updated', ['model' => __('models/questions.singular')])
        );
    }

    /**
     * Remove the specified Question from storage.
     * DELETE /questions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Question $question */
        $question = $this->questionRepository->find($id);

        if (empty($question)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questions.singular')])
            );
        }

        $question->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/questions.singular')])
        );
    }

    public function multi_delete(Request $request)
    {
        foreach ($request->ids as $id) {
            $question = $this->questionRepository->find($id);

            if (empty($question)) {
                return $this->sendError(
                    __('messages.not_found', ['model' => __('models/questions.singular')])
                );
            }

            $question->delete();
        }

        return $this->sendResponse(
            $request->ids,
            __('messages.deleted', ['model' => __('models/questions.singular')])
        );
    }
}
