<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateQuestionTypeAPIRequest;
use App\Http\Requests\API\UpdateQuestionTypeAPIRequest;
use App\Models\QuestionType;
use App\Repositories\QuestionTypeRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\QuestionTypeResource;
use Response;

/**
 * Class QuestionTypeController
 * @package App\Http\Controllers\API
 */

class QuestionTypeAPIController extends AppBaseController
{
    /** @var  QuestionTypeRepository */
    private $questionTypeRepository;

    public function __construct(QuestionTypeRepository $questionTypeRepo)
    {
        $this->questionTypeRepository = $questionTypeRepo;
    }

    /**
     * Display a listing of the QuestionType.
     * GET|HEAD /questionTypes
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $questionTypes = $this->questionTypeRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            QuestionTypeResource::collection($questionTypes),
            __('messages.retrieved', ['model' => __('models/questionTypes.plural')])
        );
    }

    /**
     * Store a newly created QuestionType in storage.
     * POST /questionTypes
     *
     * @param CreateQuestionTypeAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateQuestionTypeAPIRequest $request)
    {
        $input = $request->all();

        $input['icon'] =  $this->saveImage($request->file('icon'), 'questionTypes');

        $questionType = $this->questionTypeRepository->create($input);

        return $this->sendResponse(
            new QuestionTypeResource($questionType),
            __('messages.saved', ['model' => __('models/questionTypes.singular')])
        );
    }

    /**
     * Display the specified QuestionType.
     * GET|HEAD /questionTypes/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var QuestionType $questionType */
        $questionType = $this->questionTypeRepository->find($id);

        if (empty($questionType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questionTypes.singular')])
            );
        }

        return $this->sendResponse(
            new QuestionTypeResource($questionType),
            __('messages.retrieved', ['model' => __('models/questionTypes.singular')])
        );
    }

    /**
     * Update the specified QuestionType in storage.
     * PUT/PATCH /questionTypes/{id}
     *
     * @param int $id
     * @param UpdateQuestionTypeAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateQuestionTypeAPIRequest $request)
    {
        $input = $request->all();

        /** @var QuestionType $questionType */
        $questionType = $this->questionTypeRepository->find($id);

        if (empty($questionType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questionTypes.singular')])
            );
        }

        $input['icon'] =  $request->hasFile('icon') ?$this->updateImage($questionType->icon,$request->file('icon'), 'questionTypes'):$questionType->icon;
        $questionType = $this->questionTypeRepository->update($input, $id);

        return $this->sendResponse(
            new QuestionTypeResource($questionType),
            __('messages.updated', ['model' => __('models/questionTypes.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $questionType = $this->questionTypeRepository->find($id);
        $questionType->active = !$questionType->active;
        $questionType->save();

        return $this->sendResponse(
            new QuestionTypeResource($questionType),
            __('messages.updated', ['model' => __('models/questionTypes.singular')])
        );
    }

    /**
     * Remove the specified QuestionType from storage.
     * DELETE /questionTypes/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var QuestionType $questionType */
        $questionType = $this->questionTypeRepository->find($id);

        if (empty($questionType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/questionTypes.singular')])
            );
        }

        $questionType->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/questionTypes.singular')])
        );
    }
}
