<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateRegionAPIRequest;
use App\Http\Requests\API\UpdateRegionAPIRequest;
use App\Models\Region;
use App\Repositories\RegionRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\RegionResource;
use Response;

/**
 * Class RegionController
 * @package App\Http\Controllers\API
 */

class RegionAPIController extends AppBaseController
{
    /** @var  RegionRepository */
    private $regionRepository;

    public function __construct(RegionRepository $regionRepo)
    {
        $this->regionRepository = $regionRepo;
    }

    /**
     * Display a listing of the Region.
     * GET|HEAD /regions
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $regions = $this->regionRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            RegionResource::collection($regions),
            __('messages.retrieved', ['model' => __('models/regions.plural')])
        );
    }

    /**
     * Store a newly created Region in storage.
     * POST /regions
     *
     * @param CreateRegionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateRegionAPIRequest $request)
    {
        $input = $request->all();

        $region = $this->regionRepository->create($input);

        return $this->sendResponse(
            new RegionResource($region),
            __('messages.saved', ['model' => __('models/regions.singular')])
        );
    }

    /**
     * Display the specified Region.
     * GET|HEAD /regions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Region $region */
        $region = $this->regionRepository->find($id);

        if (empty($region)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/regions.singular')])
            );
        }

        return $this->sendResponse(
            new RegionResource($region),
            __('messages.retrieved', ['model' => __('models/regions.singular')])
        );
    }

    /**
     * Update the specified Region in storage.
     * PUT/PATCH /regions/{id}
     *
     * @param int $id
     * @param UpdateRegionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateRegionAPIRequest $request)
    {
        $input = $request->all();

        /** @var Region $region */
        $region = $this->regionRepository->find($id);

        if (empty($region)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/regions.singular')])
            );
        }

        $region = $this->regionRepository->update($input, $id);

        return $this->sendResponse(
            new RegionResource($region),
            __('messages.updated', ['model' => __('models/regions.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $region = $this->regionRepository->find($id);
        $region->active = !$region->active;
        $region->save();

        return $this->sendResponse(
            new RegionResource($region),
            __('messages.updated', ['model' => __('models/regions.singular')])
        );
    }
    /**
     * Remove the specified Region from storage.
     * DELETE /regions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Region $region */
        $region = $this->regionRepository->find($id);

        if (empty($region)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/regions.singular')])
            );
        }

        $region->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/regions.singular')])
        );
    }
}
