<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateSliderAPIRequest;
use App\Http\Requests\API\UpdateSliderAPIRequest;
use App\Models\Slider;
use App\Repositories\SliderRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\SliderResource;
use Response;

/**
 * Class SliderController
 * @package App\Http\Controllers\API
 */

class SliderAPIController extends AppBaseController
{
    /** @var  SliderRepository */
    private $sliderRepository;

    public function __construct(SliderRepository $sliderRepo)
    {
        $this->sliderRepository = $sliderRepo;
    }

    /**
     * Display a listing of the Slider.
     * GET|HEAD /sliders
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $sliders = $this->sliderRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('type',$request->get('type'));

        return $this->sendResponse(
            SliderResource::collection($sliders),
            __('messages.retrieved', ['model' => __('models/sliders.plural')])
        );
    }

    /**
     * Store a newly created Slider in storage.
     * POST /sliders
     *
     * @param CreateSliderAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateSliderAPIRequest $request)
    {
        $input = $request->all();

        $input['image'] = $this->saveImage($request->file('image'), 'sliders');

        $slider = $this->sliderRepository->create($input);

        return $this->sendResponse(
            new SliderResource($slider),
            __('messages.saved', ['model' => __('models/sliders.singular')])
        );
    }

    /**
     * Display the specified Slider.
     * GET|HEAD /sliders/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Slider $slider */
        $slider = $this->sliderRepository->find($id);

        if (empty($slider)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/sliders.singular')])
            );
        }

        return $this->sendResponse(
            new SliderResource($slider),
            __('messages.retrieved', ['model' => __('models/sliders.singular')])
        );
    }

    /**
     * Update the specified Slider in storage.
     * PUT/PATCH /sliders/{id}
     *
     * @param int $id
     * @param UpdateSliderAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateSliderAPIRequest $request)
    {
        $input = $request->all();

        /** @var Slider $slider */
        $slider = $this->sliderRepository->find($id);

        if (empty($slider)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/sliders.singular')])
            );
        }

        $input['image'] = $request->hasFile('image') ?$this->updateImage($slider->image,$request->file('image'), 'sliders'):$slider->image;
        $slider = $this->sliderRepository->update($input, $id);

        return $this->sendResponse(
            new SliderResource($slider),
            __('messages.updated', ['model' => __('models/sliders.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $slider = $this->sliderRepository->find($id);
        $slider->active = !$slider->active;
        $slider->save();

        return $this->sendResponse(
            new SliderResource($slider),
            __('messages.updated', ['model' => __('models/sliders.singular')])
        );
    }

    /**
     * Remove the specified Slider from storage.
     * DELETE /sliders/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Slider $slider */
        $slider = $this->sliderRepository->find($id);

        if (empty($slider)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/sliders.singular')])
            );
        }

        $slider->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/sliders.singular')])
        );
    }
}
