<?php

namespace App\Http\Controllers\API;

use App\Http\Resources\MyChampionshipResource;
use App\Http\Resources\StatCompetitionResource;
use App\Http\Resources\StatTrainingResource;
use App\Http\Resources\StudentResource;
use App\Http\Resources\TypeLevelResource;
use App\Models\Championship;
use App\Models\Competition;
use App\Models\Favourite;
use App\Models\Training;
use App\Models\User;
use App\Repositories\UserRepository;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use Response;

/**
 * Class UserController
 * @package App\Http\Controllers\API
 */

class StudentAPIController extends AppBaseController
{
    /** @var  UserRepository */
    private $userRepository;

    public function __construct(UserRepository $userRepo)
    {
        $this->userRepository = $userRepo;
    }

    /**
     * Display a listing of the User.
     * GET|HEAD /users
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $users = User::where('role','student')->where('id','<>', auth('sanctum')->id());

        if ($request->get('country_id')){
            $users->where('country_id',$request->get('country_id'));
        }

        if ($request->get('city_id')){
            $users->where('city_id',$request->get('city_id'));
        }

        if ($request->get('region_id')){
            $users->where('region_id',$request->get('region_id'));
        }

        if ($request->get('search')){
            $users->where('name', 'like', '%' . request('search') . '%')->orWhere('phone', 'like', '%' . request('search') . '%');
        }

        if ($request->get('online')){
            $users->where('online',$request->get('online'));
        }

        if ($request->get('special')){
            $users->whereIn('id',Favourite::pluck('friend_id')->toArray());
        }

        if ($request->get('last_login_users')){
            $users->whereNotNull('last_login_at')->orderBy('last_login_at', 'desc');
        }

        if ($request->get('most_students')){
            $users->where('role', 'student')->whereNotNull('student_points')->orderBy('student_points', 'desc');
        }

        if ($request->get('subscription_students')){
            $users->where('role', 'student')->whereHas('subscription', fn($query) => $query->where('active', 1));
        }

        if ($request->get('most_trainings_students')){
            $users->where('role', 'student')->whereHas('trainings', fn($query) => $query->where('finish', 1));
        }

        if (request('rows')){
            $users = $users->paginate(request('rows'));
        }
        else{
            $users = $users->get();
        }


        if ($request->user()->role =="admin"||$request->user()->role =="sub_admin") {
            return $this->sendResponse(
                \App\Http\Resources\admin\StudentResource::collection($users),
                __('messages.retrieved', ['model' => __('models/users.plural')])
            );
        }


        return $this->sendResponse(
            StudentResource::collection($users),
            __('messages.retrieved', ['model' => __('models/users.plural')])
        );
    }


    public function statistics($id): JsonResponse
    {
        $data = [];

        $trainings = Training::where('finish',1)->where('user_id',$id)->get();

        $competitions = Competition::where('finish',1)->where('user_id',$id)->orWhereHas('subscribers', fn ($query) => $query->where('user_id', $id))->get();

        $championships = Championship::where('finish',1)->whereHas('subscribers', fn ($query) => $query->where('user_id', $id))->get();

        if (request('date_from')&&request('date_to') ){
            $from = date(request('date_from'));
            $to = date(request('date_to'));

            $trainings = $trainings->whereBetween('created_at', [$from, $to]);
            $competitions = $competitions->whereBetween('competition_datetime', [$from, $to]);
            $championships = $championships->whereBetween('start_at', [$from, $to]);
        }

        $data['trainings_count'] = count($trainings);
        $data['competitions_count'] = count($competitions);
        $data['championships_count'] = count($championships);


        $data['trainings'] =  StatTrainingResource::collection($trainings);
        $data['competitions'] =  StatCompetitionResource::collection($competitions);
        $data['championships'] =  MyChampionshipResource::collection($championships);

        return $this->sendResponse(
            $data,
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );

    }

    public function friends($id): JsonResponse
    {
        $users = User::find($id)->friends();

        if (\request('search')){
            $users->where('name', 'like', '%' . request('search') . '%');
        }

        return $this->sendResponse(data: StudentResource::collection($users->get()));
    }

    public function levels($id): JsonResponse
    {
        return $this->sendResponse(data: TypeLevelResource::collection(User::find($id)->levels));
    }

    public function show($id)
    {
        /** @var User $user */
        $user = $this->userRepository->find($id);

        if (empty($user)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/users.singular')])
            );
        }

        if (auth('sanctum')->user()->role =="admin") {
            return $this->sendResponse(
                new \App\Http\Resources\admin\StudentDetailsResource($user),
                __('messages.retrieved', ['model' => __('models/users.plural')])
            );
        }

        return $this->sendResponse(
            new StudentResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );
    }


}
