<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateSubscriptionAPIRequest;
use App\Http\Requests\API\UpdateSubscriptionAPIRequest;
use App\Models\Subscription;
use App\Repositories\SubscriptionRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\SubscriptionResource;
use Response;

/**
 * Class SubscriptionController
 * @package App\Http\Controllers\API
 */

class SubscriptionAPIController extends AppBaseController
{
    /** @var  SubscriptionRepository */
    private $subscriptionRepository;

    public function __construct(SubscriptionRepository $subscriptionRepo)
    {
        $this->subscriptionRepository = $subscriptionRepo;
    }

    /**
     * Display a listing of the Subscription.
     * GET|HEAD /subscriptions
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $subscriptions = $this->subscriptionRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            SubscriptionResource::collection($subscriptions),
            __('messages.retrieved', ['model' => __('models/subscriptions.plural')])
        );
    }

    /**
     * Store a newly created Subscription in storage.
     * POST /subscriptions
     *
     * @param CreateSubscriptionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateSubscriptionAPIRequest $request)
    {
        $input = $request->all();

        $subscription = $this->subscriptionRepository->create($input);

        return $this->sendResponse(
            new SubscriptionResource($subscription),
            __('messages.saved', ['model' => __('models/subscriptions.singular')])
        );
    }




    /**
     * Display the specified Subscription.
     * GET|HEAD /subscriptions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Subscription $subscription */
        $subscription = $this->subscriptionRepository->find($id);

        if (empty($subscription)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/subscriptions.singular')])
            );
        }

        return $this->sendResponse(
            new SubscriptionResource($subscription),
            __('messages.retrieved', ['model' => __('models/subscriptions.singular')])
        );
    }

    /**
     * Update the specified Subscription in storage.
     * PUT/PATCH /subscriptions/{id}
     *
     * @param int $id
     * @param UpdateSubscriptionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateSubscriptionAPIRequest $request)
    {
        $input = $request->all();

        /** @var Subscription $subscription */
        $subscription = $this->subscriptionRepository->find($id);

        if (empty($subscription)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/subscriptions.singular')])
            );
        }

        $subscription = $this->subscriptionRepository->update($input, $id);

        return $this->sendResponse(
            new SubscriptionResource($subscription),
            __('messages.updated', ['model' => __('models/subscriptions.singular')])
        );
    }

    /**
     * Remove the specified Subscription from storage.
     * DELETE /subscriptions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Subscription $subscription */
        $subscription = $this->subscriptionRepository->find($id);

        if (empty($subscription)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/subscriptions.singular')])
            );
        }

        $subscription->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/subscriptions.singular')])
        );
    }
}
