<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateTrainerGalleryAPIRequest;
use App\Http\Requests\API\UpdateTrainerGalleryAPIRequest;
use App\Models\TrainerGallery;
use App\Repositories\TrainerGalleryRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TrainerGalleryResource;
use Response;

/**
 * Class TrainerGalleryController
 * @package App\Http\Controllers\API
 */

class TrainerGalleryAPIController extends AppBaseController
{
    /** @var  TrainerGalleryRepository */
    private $trainerGalleryRepository;

    public function __construct(TrainerGalleryRepository $trainerGalleryRepo)
    {
        $this->trainerGalleryRepository = $trainerGalleryRepo;
    }

    /**
     * Display a listing of the TrainerGallery.
     * GET|HEAD /trainerGalleries
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $trainerGalleries = $this->trainerGalleryRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if (auth('sanctum')->check() && $request->user()?->role =='trainer') {
            $trainerGalleries = $trainerGalleries->where('user_id', auth('sanctum')->id());
        }

        if ($request->get('trainer_id')){
            $trainerGalleries = $trainerGalleries->where('user_id',$request->trainer_id);
        }

        if ($request->get('type')){
            $trainerGalleries = $trainerGalleries->where('type',$request->type);
        }

        return $this->sendResponse(
            TrainerGalleryResource::collection($trainerGalleries),
            __('messages.retrieved', ['model' => __('models/trainerGalleries.plural')])
        );
    }

    /**
     * Store a newly created TrainerGallery in storage.
     * POST /trainerGalleries
     *
     * @param CreateTrainerGalleryAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTrainerGalleryAPIRequest $request)
    {
        $input = $request->all();

        if ($request->user()->role == 'trainer') {
            $input['user_id'] = auth('sanctum')->id();
        }

        else{
            $input['user_id'] = $request->trainer_id;
        }

        $input['file'] = $this->saveImage($request->file('file'), 'trainerGalleries');

        $trainerGallery = $this->trainerGalleryRepository->create($input);

        return $this->sendResponse(
            new TrainerGalleryResource($trainerGallery),
            __('messages.saved', ['model' => __('models/trainerGalleries.singular')])
        );
    }

    /**
     * Display the specified TrainerGallery.
     * GET|HEAD /trainerGalleries/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var TrainerGallery $trainerGallery */
        $trainerGallery = $this->trainerGalleryRepository->find($id);

        if (empty($trainerGallery)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerGalleries.singular')])
            );
        }

        return $this->sendResponse(
            new TrainerGalleryResource($trainerGallery),
            __('messages.retrieved', ['model' => __('models/trainerGalleries.singular')])
        );
    }

    /**
     * Update the specified TrainerGallery in storage.
     * PUT/PATCH /trainerGalleries/{id}
     *
     * @param int $id
     * @param UpdateTrainerGalleryAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateTrainerGalleryAPIRequest $request)
    {
        $input = $request->all();

        /** @var TrainerGallery $trainerGallery */
        $trainerGallery = $this->trainerGalleryRepository->find($id);

        if (empty($trainerGallery)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerGalleries.singular')])
            );
        }

        $trainerGallery = $this->trainerGalleryRepository->update($input, $id);

        return $this->sendResponse(
            new TrainerGalleryResource($trainerGallery),
            __('messages.updated', ['model' => __('models/trainerGalleries.singular')])
        );
    }

    /**
     * Remove the specified TrainerGallery from storage.
     * DELETE /trainerGalleries/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var TrainerGallery $trainerGallery */
        $trainerGallery = $this->trainerGalleryRepository->find($id);

        if (empty($trainerGallery)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerGalleries.singular')])
            );
        }

        $trainerGallery->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/trainerGalleries.singular')])
        );
    }
}
