<?php

namespace App\Http\Controllers\API;

use App\Events\MyEvent;
use App\Http\Requests\API\CreateTrainerMessageAPIRequest;
use App\Http\Requests\API\UpdateTrainerMessageAPIRequest;
use App\Http\Resources\StudentChatResource;
use App\Http\Resources\TrainerChatResource;
use App\Models\TrainerMessage;
use App\Models\User;
use App\Repositories\TrainerMessageRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TrainerMessageResource;
use Response;
use Validator;
/**
 * Class TrainerMessageController
 * @package App\Http\Controllers\API
 */

class TrainerMessageAPIController extends AppBaseController
{
    /** @var  TrainerMessageRepository */
    private $trainerMessageRepository;

    public function __construct(TrainerMessageRepository $trainerMessageRepo)
    {
        $this->trainerMessageRepository = $trainerMessageRepo;
    }

    /**
     * Display a listing of the TrainerMessage.
     * GET|HEAD /trainerMessages
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {

        $trainerMessages = $this->trainerMessageRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if($request->user()->role =='trainer'){
            $trainerMessages = $trainerMessages->where('trainer_id',$request->user()->id);
        }

        if($request->user()->role =='student'){
            $trainerMessages = $trainerMessages->where('student_id',$request->user()->id);
        }

        if ($request->get('trainer_id')){
            $trainerMessages = $trainerMessages->where('trainer_id',$request->get('trainer_id'));
        }

        if ($request->get('student_id')){
            $trainerMessages = $trainerMessages->where('student_id',$request->get('student_id'));
        }



        return $this->sendResponse(
            TrainerMessageResource::collection($trainerMessages),
            __('messages.retrieved', ['model' => __('models/trainerMessages.plural')])
        );
    }

    public function chat(Request $request)
    {

        if($request->user()->role =='trainer'){

            $messages = collect(TrainerMessage::where('trainer_id',$request->user()->id)->whereNotNull('student_id')->orderBy('created_at','desc')->get());

            $messagesUnique = $messages->unique('student_id');

            $messagesUnique->values()->all();

            return $this->sendResponse(
                TrainerChatResource::collection($messagesUnique),
                __('messages.retrieved', ['model' => __('models/trainerMessages.plural')])
            );

        }

        if($request->user()->role =='admin'){

            $messages = collect(TrainerMessage::where('trainer_id',$request->trainer_id)->whereNotNull('student_id')->orderBy('created_at','desc')->get());

            $messagesUnique = $messages->unique('student_id');

            $messagesUnique->values()->all();

            return $this->sendResponse(
                TrainerChatResource::collection($messagesUnique),
                __('messages.retrieved', ['model' => __('models/trainerMessages.plural')])
            );

        }

        if($request->user()->role =='student'){

                $messages = collect(TrainerMessage::where('student_id',$request->user()->id)->whereNotNull('trainer_id')->orderBy('created_at','asc')->get());

                $messagesUnique = $messages->unique('trainer_id');

                $messagesUnique->values()->all();

                return $this->sendResponse(
                    StudentChatResource::collection($messagesUnique),
                    __('messages.retrieved', ['model' => __('models/trainerMessages.plural')])
                );

        }

    }

    /**
     * Store a newly created TrainerMessage in storage.
     * POST /trainerMessages
     *
     * @param CreateTrainerMessageAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTrainerMessageAPIRequest $request)
    {
        $input = $request->all();

        if($request->user()->role =='student'){
            $validator = Validator::make($request->all(), [
                'trainer_id' => 'required||exists:users,id',
            ]);
            if ($validator->fails()) {
                return $this->sendError($validator->errors()->first(),$validator->errors());
            }
            $input['student_id'] = auth('sanctum')->id();
            $input['sender_type'] = 'student';
        }

        if($request->user()->role =='trainer'){
            $validator = Validator::make($request->all(), [
                'student_id' => 'required||exists:users,id',
            ]);
            if ($validator->fails()) {
                return $this->sendError($validator->errors()->first(),$validator->errors());
            }
            $input['trainer_id'] = auth('sanctum')->id();
            $input['sender_type'] = 'trainer';
        }

        if($request->file('file')){
            $input['file'] = $this->saveImage($request->file('file'), 'trainerMessages');
        }

        $trainerMessage = $this->trainerMessageRepository->create($input);

        event(new \App\Events\TrainerMessage(new TrainerMessageResource($trainerMessage)));

        return $this->sendResponse(
            new TrainerMessageResource($trainerMessage),
            __('messages.saved', ['model' => __('models/trainerMessages.singular')])
        );
    }

    /**
     * Display the specified TrainerMessage.
     * GET|HEAD /trainerMessages/{id}
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        /** @var TrainerMessage $trainerMessage */
        $trainerMessage = $this->trainerMessageRepository->find($id);

        if (empty($trainerMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerMessages.singular')])
            );
        }

        return $this->sendResponse(
            new TrainerMessageResource($trainerMessage),
            __('messages.retrieved', ['model' => __('models/trainerMessages.singular')])
        );
    }

    /**
     * Update the specified TrainerMessage in storage.
     * PUT/PATCH /trainerMessages/{id}
     *
     * @param int $id
     * @param UpdateTrainerMessageAPIRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateTrainerMessageAPIRequest $request)
    {
        $input = $request->all();

        /** @var TrainerMessage $trainerMessage */
        $trainerMessage = $this->trainerMessageRepository->find($id);

        if (empty($trainerMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerMessages.singular')])
            );
        }

        $trainerMessage = $this->trainerMessageRepository->update($input, $id);

        return $this->sendResponse(
            new TrainerMessageResource($trainerMessage),
            __('messages.updated', ['model' => __('models/trainerMessages.singular')])
        );
    }

    /**
     * Remove the specified TrainerMessage from storage.
     * DELETE /trainerMessages/{id}
     *
     * @param int $id
     *
     * @throws \Exception
     *
     * @return Response
     */
    public function destroy($id)
    {
        /** @var TrainerMessage $trainerMessage */
        $trainerMessage = $this->trainerMessageRepository->find($id);

        if (empty($trainerMessage)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerMessages.singular')])
            );
        }

        $trainerMessage->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/trainerMessages.singular')])
        );
    }
}
