<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateTrainerRegionAPIRequest;
use App\Http\Requests\API\UpdateTrainerRegionAPIRequest;
use App\Models\TrainerRegion;
use App\Repositories\TrainerRegionRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TrainerRegionResource;
use Response;

/**
 * Class TrainerRegionController
 * @package App\Http\Controllers\API
 */

class TrainerRegionAPIController extends AppBaseController
{
    /** @var  TrainerRegionRepository */
    private $trainerRegionRepository;

    public function __construct(TrainerRegionRepository $trainerRegionRepo)
    {
        $this->trainerRegionRepository = $trainerRegionRepo;
    }

    /**
     * Display a listing of the TrainerRegion.
     * GET|HEAD /trainerRegions
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $trainerRegions = $this->trainerRegionRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if (auth('sanctum')->check() && $request->user()?->role =='trainer') {
            $trainerRegions = $trainerRegions->where('user_id', auth('sanctum')->id());
        }

        if ($request->get('trainer_id')){
            $trainerRegions = $trainerRegions->where('user_id',$request->trainer_id);
        }

        return $this->sendResponse(
            TrainerRegionResource::collection($trainerRegions),
            __('messages.retrieved', ['model' => __('models/trainerRegions.plural')])
        );
    }

    /**
     * Store a newly created TrainerRegion in storage.
     * POST /trainerRegions
     *
     * @param CreateTrainerRegionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTrainerRegionAPIRequest $request)
    {
        $input = $request->all();

        if ($request->user()->role == 'trainer') {
            $input['user_id'] = auth('sanctum')->id();
        }

        else{
            $input['user_id'] = $request->trainer_id;
        }

        $trainerRegion = $this->trainerRegionRepository->create($input);

        return $this->sendResponse(
            new TrainerRegionResource($trainerRegion),
            __('messages.saved', ['model' => __('models/trainerRegions.singular')])
        );
    }

    /**
     * Display the specified TrainerRegion.
     * GET|HEAD /trainerRegions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var TrainerRegion $trainerRegion */
        $trainerRegion = $this->trainerRegionRepository->find($id);

        if (empty($trainerRegion)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerRegions.singular')])
            );
        }

        return $this->sendResponse(
            new TrainerRegionResource($trainerRegion),
            __('messages.retrieved', ['model' => __('models/trainerRegions.singular')])
        );
    }

    /**
     * Update the specified TrainerRegion in storage.
     * PUT/PATCH /trainerRegions/{id}
     *
     * @param int $id
     * @param UpdateTrainerRegionAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateTrainerRegionAPIRequest $request)
    {
        $input = $request->all();

        /** @var TrainerRegion $trainerRegion */
        $trainerRegion = $this->trainerRegionRepository->find($id);

        if (empty($trainerRegion)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerRegions.singular')])
            );
        }

        $trainerRegion = $this->trainerRegionRepository->update($input, $id);

        return $this->sendResponse(
            new TrainerRegionResource($trainerRegion),
            __('messages.updated', ['model' => __('models/trainerRegions.singular')])
        );
    }

    /**
     * Remove the specified TrainerRegion from storage.
     * DELETE /trainerRegions/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var TrainerRegion $trainerRegion */
        $trainerRegion = $this->trainerRegionRepository->find($id);

        if (empty($trainerRegion)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainerRegions.singular')])
            );
        }

        $trainerRegion->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/trainerRegions.singular')])
        );
    }
}
