<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateTrainingAPIRequest;
use App\Http\Requests\API\UpdateTrainingAPIRequest;
use App\Http\Resources\QuestionResource;
use App\Http\Resources\TrainingQuestionResource;
use App\Models\Models\TrainingQuestion;
use App\Models\Question;
use App\Models\Setting;
use App\Models\Training;
use App\Models\User;
use App\Repositories\TrainingRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TrainingResource;
use Carbon\Carbon;
use Response;
use Validator;
/**
 * Class TrainingController
 * @package App\Http\Controllers\API
 */

class TrainingAPIController extends AppBaseController
{
    /** @var  TrainingRepository */
    private $trainingRepository;

    public function __construct(TrainingRepository $trainingRepo)
    {
        $this->trainingRepository = $trainingRepo;

        $this->middleware('subscribed')->only(['store','add_training_by_trainer_for_students','share_training_by_trainer_for_students']);
    }

    /**
     * Display a listing of the Training.
     * GET|HEAD /trainings
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $trainings = $this->trainingRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if ($request->user()?->role =='student'){
            $trainings = $trainings->where('user_id',auth('sanctum')->id());
        }

        if ($request->user()?->role =='trainer'){
            $trainings = $trainings->where('trainer_id',auth('sanctum')->id());
        }

        if ($request->get('trainer_id')){
            $trainings = $trainings->where('trainer_id',$request->trainer_id);
        }

        if ($request->get('student_id')){
            $trainings = $trainings->where('user_id',$request->student_id);
        }

        $trainings = $trainings->sortByDesc('created_at');

        return $this->sendResponse(
            TrainingResource::collection($trainings),
            __('messages.retrieved', ['model' => __('models/trainings.plural')])
        );
    }

    /**
     * Store a newly created Training in storage.
     * POST /trainings
     *
     * @param CreateTrainingAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTrainingAPIRequest $request)
    {
        $input = $request->all();

        $input['user_id'] = auth('sanctum')->id();

        if(isset($request->training_datetime)){
            $input['expired_at'] = Carbon::parse($request->training_datetime)->addMinutes($request->training_period)->timezone('Africa/Cairo');
        }


        $training = $this->trainingRepository->create($input);

        $data = [];

        $all_questions = $this->trainingRepository->get_questions($training);


        if(is_countable($all_questions) == false){
            return  $all_questions ;
        }

        //$training->questions()->sync($all_questions->pluck('id')->toArray());

        $data['training'] =  new TrainingResource($training);
        $data['questions'] = QuestionResource::collection($all_questions);


    //     if ($request->user()?->role =='trainer' && $request->subscribers){
    //     foreach ($request->subscribers as $subscriber_id){
    //         $new_training = $training->replicate();
    //         $new_training->user_id = $subscriber_id;
    //         $new_training->active = 0;
    //         $new_training->training_id = $training->id;
    //         $new_training->save();

    //         $new_training->questions()->sync($all_questions->pluck('id')->toArray());

    //         $user = User::find($subscriber_id);
    //         if($user){
    //             $user->notify(new \App\Notifications\Training($new_training));

    //             sendFirebaseNotification($user->token,trans('messages.training_invitation'),new \App\Notifications\Training($new_training));
    //         }

    //     }
    // }



        return $this->sendResponse(
            $data,
            __('messages.saved', ['model' => __('models/trainings.singular')])
        );
    }

 public function add_training_by_trainer_for_students(CreateTrainingAPIRequest $request): \Illuminate\Http\JsonResponse
 {
    $input = $request->all();

    $input['user_id'] = auth('sanctum')->id();
    $input['trainer_id'] = auth('sanctum')->id();
    if(isset($request->training_datetime)){
        $input['expired_at'] = Carbon::parse($request->training_datetime)->addMinutes($request->training_period)->timezone('Africa/Cairo');
    }

    $training = $this->trainingRepository->create($input);


    $all_questions = $this->trainingRepository->get_questions($training);

    $training->questions()->sync($all_questions->pluck('id')->toArray());

    if (count($all_questions) ==0){
        $training->delete();
        return $this->sendError('No Questions Available for this setting');
    }

        if ($request->user()?->role =='trainer' && $request->subscribers){
        foreach ($request->subscribers as $subscriber_id){
            $new_training = $training->replicate();
            $new_training->user_id = $subscriber_id;
            $new_training->active = 0;
            $new_training->training_id = $training->id;
            $new_training->save();

            $new_training->questions()->sync($all_questions->pluck('id')->toArray());

            $user = User::find($subscriber_id);
            if($user){
                $user->notify(new \App\Notifications\Training($new_training));

                sendFirebaseNotification($user->token,trans('messages.training_invitation'),$new_training->trainer?->name);
            }

        }
    }

     return $this->sendResponse(
         new TrainingResource($training),
         __('messages.saved', ['model' => __('models/trainings.singular')])
     );

    }

    public function share_training_by_trainer_for_students($id, Request $request): \Illuminate\Http\JsonResponse
    {
        $old_training = $this->trainingRepository->find($id);

        foreach ($request->subscribers as $subscriber_id){
            $new_training = $old_training->replicate();
            $new_training->user_id = $subscriber_id;
            $new_training->active = 0;
            $new_training->training_id = $old_training->id;
            $new_training->save();

            $new_training->questions()->sync($this->trainingRepository->get_questions($new_training)->pluck('id')->toArray());

            $user = User::find($subscriber_id);
            if($user){
                $user->notify(new \App\Notifications\Training($new_training));

                sendFirebaseNotification($user->token,trans('messages.training_invitation'),$new_training->trainer?->name);
            }


        }


        return $this->sendResponse(
            new TrainingResource($old_training),
            __('messages.saved', ['model' => __('models/trainings.singular')])
        );
    }

    /**
     * Display the specified Training.
     * GET|HEAD /trainings/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var Training $training */
        $training = $this->trainingRepository->find($id);

        if (empty($training)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainings.singular')])
            );
        }

        $data['training'] =  new TrainingResource($training);
        $data['questions'] = QuestionResource::collection($training->questions);


        return $this->sendResponse(
            $data,
            __('messages.saved', ['model' => __('models/trainings.singular')])
        );
    }



    /**
     * Update the specified Training in storage.
     * PUT/PATCH /trainings/{id}
     *
     * @param int $id
     * @param UpdateTrainingAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateTrainingAPIRequest $request)
    {
        $input = $request->all();

        /** @var Training $training */
        $training = $this->trainingRepository->find($id);

        if (empty($training)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainings.singular')])
            );
        }

        if ($request->correct_answers_count){
            $input['student_points'] = $request->correct_answers_count * Setting::first()->training_points ;
        }

        if ($request->answering_period && $request->correct_answers_count){
            $input['supposed_solved'] = exam_equation($training->training_period * 60 ,$request->answering_period * 60,$request->correct_answers_count) ;
        }

        $training = $this->trainingRepository->update($input, $id);

        return $this->sendResponse(
            new TrainingResource($training),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $training = $this->trainingRepository->find($id);
        $training->active = !$training->active;
        $training->save();

        return $this->sendResponse(
            new TrainingResource($training),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    public function collectStudentPoints($id): \Illuminate\Http\JsonResponse
    {
        $training = $this->trainingRepository->find($id);
        if ( $training->student_points_is_collected == 0){
            $training->student_points_is_collected = 1;
            $training->save();

            $user = auth('sanctum')->user();
            $user->student_points = $user->student_points + $training->student_points;
            $user->save();
        }
        return $this->sendResponse(
            new TrainingResource($training),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    public function answer_question_training()
    {
       $answer_question = TrainingQuestion::where('training_id',\request('training_id'))->where('question_id',\request('question_id'))->first();
       $answer_question->is_correct = \request('is_correct');
        $answer_question->wrong_answer = \request('wrong_answer');
        $answer_question->save();

        return $this->sendResponse(
            new TrainingQuestionResource($answer_question),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    public function answer_training_questions()
    {
        $validator = Validator::make(\request()->all(), [
            'questions' => 'required|array',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors()->first());
        }
        $training = $this->trainingRepository->find(\request('training_id'));
        if (empty($training)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainings.singular')])
            );
        }


        foreach (\request('questions') as $i=> $question_id){
            $answer_question = TrainingQuestion::where('training_id',\request('training_id'))->where('question_id',$question_id)->firstorFail();
            $answer_question->is_correct = \request('is_correct')[$i];
            $answer_question->wrong_answer = \request('wrong_answer')[$i];
            $answer_question->save();
        }


        return $this->sendResponse(
            new TrainingResource($training),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    public function training_questions_answers(): \Illuminate\Http\JsonResponse
    {
        $training_questions_answers = TrainingQuestion::where('training_id',\request('training_id'))->get();

        return $this->sendResponse(
             TrainingQuestionResource::collection($training_questions_answers),
            __('messages.updated', ['model' => __('models/trainings.singular')])
        );
    }

    /**
     * Remove the specified Training from storage.
     * DELETE /trainings/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var Training $training */
        $training = $this->trainingRepository->find($id);

        if (empty($training)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainings.singular')])
            );
        }

        $training->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/trainings.singular')])
        );
    }
}
