<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateTrainingTypeAPIRequest;
use App\Http\Requests\API\UpdateTrainingTypeAPIRequest;
use App\Http\Resources\TrainingTypeDetailsResource;
use App\Models\TrainingType;
use App\Repositories\TrainingTypeRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TrainingTypeResource;
use Response;

/**
 * Class TrainingTypeController
 * @package App\Http\Controllers\API
 */

class TrainingTypeAPIController extends AppBaseController
{
    /** @var  TrainingTypeRepository */
    private $trainingTypeRepository;

    public function __construct(TrainingTypeRepository $trainingTypeRepo)
    {
        $this->trainingTypeRepository = $trainingTypeRepo;
    }

    /**
     * Display a listing of the TrainingType.
     * GET|HEAD /trainingTypes
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request): \Illuminate\Http\JsonResponse
    {
        $trainingTypes = $this->trainingTypeRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        return $this->sendResponse(
            TrainingTypeResource::collection($trainingTypes),
            __('messages.retrieved', ['model' => __('models/trainingTypes.plural')])
        );
    }

    public function training_types_tree(): \Illuminate\Http\JsonResponse
    {
        $trainingTypes = $this->trainingTypeRepository->all();

        if (\request('training_type_id')){
            $trainingTypes->where('id',\request('training_type_id'));
        }

        return $this->sendResponse(
            TrainingTypeDetailsResource::collection($trainingTypes),
            __('messages.retrieved', ['model' => __('models/trainingTypes.plural')])
        );
    }

    /**
     * Store a newly created TrainingType in storage.
     * POST /trainingTypes
     *
     * @param CreateTrainingTypeAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTrainingTypeAPIRequest $request)
    {
        $trainingType = TrainingType::where('title','LIKE',json_encode($request->title,JSON_UNESCAPED_UNICODE ))->first();
        if ($trainingType == null){
            $input = $request->all();

            //$input['icon'] =  $this->saveImage($request->file('icon'), 'trainingTypes');

            $trainingType = $this->trainingTypeRepository->create($input);
        }
        return $this->sendResponse(
            new TrainingTypeResource($trainingType),
            __('messages.saved', ['model' => __('models/trainingTypes.singular')])
        );
    }

    /**
     * Display the specified TrainingType.
     * GET|HEAD /trainingTypes/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var TrainingType $trainingType */
        $trainingType = $this->trainingTypeRepository->find($id);

        if (empty($trainingType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainingTypes.singular')])
            );
        }

        return $this->sendResponse(
            new TrainingTypeResource($trainingType),
            __('messages.retrieved', ['model' => __('models/trainingTypes.singular')])
        );
    }

    /**
     * Update the specified TrainingType in storage.
     * PUT/PATCH /trainingTypes/{id}
     *
     * @param int $id
     * @param UpdateTrainingTypeAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateTrainingTypeAPIRequest $request)
    {
        $input = $request->all();

        /** @var TrainingType $trainingType */
        $trainingType = $this->trainingTypeRepository->find($id);

        if (empty($trainingType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainingTypes.singular')])
            );
        }

        //$input['icon'] =  $request->hasFile('icon') ?$this->updateImage($trainingType->icon,$request->file('icon'), 'trainingTypes'):$trainingType->icon;
        $trainingType = $this->trainingTypeRepository->update($input, $id);

        return $this->sendResponse(
            new TrainingTypeResource($trainingType),
            __('messages.updated', ['model' => __('models/trainingTypes.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $trainingType = $this->trainingTypeRepository->find($id);
        $trainingType->active = !$trainingType->active;
        $trainingType->save();

        return $this->sendResponse(
            new TrainingTypeResource($trainingType),
            __('messages.updated', ['model' => __('models/trainingTypes.singular')])
        );
    }

    /**
     * Remove the specified TrainingType from storage.
     * DELETE /trainingTypes/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var TrainingType $trainingType */
        $trainingType = $this->trainingTypeRepository->find($id);

        if (empty($trainingType)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/trainingTypes.singular')])
            );
        }

        $trainingType->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/trainingTypes.singular')])
        );
    }
}
