<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateTypeLevelAPIRequest;
use App\Http\Requests\API\UpdateTypeLevelAPIRequest;
use App\Http\Resources\ChampionshipLevelResource;
use App\Http\Resources\CompetitionLevelResource;
use App\Models\TypeLevel;
use App\Repositories\TypeLevelRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\TypeLevelResource;
use Response;

/**
 * Class TypeLevelController
 * @package App\Http\Controllers\API
 */

class TypeLevelAPIController extends AppBaseController
{
    /** @var  TypeLevelRepository */
    private $typeLevelRepository;

    public function __construct(TypeLevelRepository $typeLevelRepo)
    {
        $this->typeLevelRepository = $typeLevelRepo;
    }

    /**
     * Display a listing of the TypeLevel.
     * GET|HEAD /typeLevels
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $typeLevels = $this->typeLevelRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        );

        if (\request('training_type_id')){
            $typeLevels->where('training_type_id',\request('training_type_id'));
        }

        return $this->sendResponse(
            TypeLevelResource::collection($typeLevels),
            __('messages.retrieved', ['model' => __('models/typeLevels.plural')])
        );
    }


    public function competition(Request $request)
    {
        $typeLevels = $this->typeLevelRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('training_type_id',2);


        return $this->sendResponse(
            CompetitionLevelResource::collection($typeLevels),
            __('messages.retrieved', ['model' => __('models/typeLevels.plural')])
        );
    }


    public function championship(Request $request)
    {
        $typeLevels = $this->typeLevelRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('training_type_id',2);


        return $this->sendResponse(
            ChampionshipLevelResource::collection($typeLevels),
            __('messages.retrieved', ['model' => __('models/typeLevels.plural')])
        );
    }

    public function personal_levels(Request $request): \Illuminate\Http\JsonResponse
    {
        $typeLevels = $this->typeLevelRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('training_type_id',2);


        return $this->sendResponse(
            TypeLevelResource::collection($typeLevels),
            __('messages.retrieved', ['model' => __('models/typeLevels.plural')])
        );
    }
    /**
     * Store a newly created TypeLevel in storage.
     * POST /typeLevels
     *
     * @param CreateTypeLevelAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateTypeLevelAPIRequest $request)
    {
        $typeLevel = TypeLevel::where('title','LIKE',json_encode($request->title,JSON_UNESCAPED_UNICODE ))->where('training_type_id',$request->training_type_id)->first();
        if ($typeLevel == null) {
            $input = $request->all();

            if ($request->file('icon')){
               $input['icon'] = $this->saveImage($request->file('icon'), 'typeLevels');
            }

            $typeLevel = $this->typeLevelRepository->create($input);
        }

        return $this->sendResponse(
            new TypeLevelResource($typeLevel),
            __('messages.saved', ['model' => __('models/typeLevels.singular')])
        );
    }

    /**
     * Display the specified TypeLevel.
     * GET|HEAD /typeLevels/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var TypeLevel $typeLevel */
        $typeLevel = $this->typeLevelRepository->find($id);

        if (empty($typeLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/typeLevels.singular')])
            );
        }

        return $this->sendResponse(
            new TypeLevelResource($typeLevel),
            __('messages.retrieved', ['model' => __('models/typeLevels.singular')])
        );
    }

    /**
     * Update the specified TypeLevel in storage.
     * PUT/PATCH /typeLevels/{id}
     *
     * @param int $id
     * @param UpdateTypeLevelAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateTypeLevelAPIRequest $request)
    {
        $input = $request->all();

        /** @var TypeLevel $typeLevel */
        $typeLevel = $this->typeLevelRepository->find($id);

        if (empty($typeLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/typeLevels.singular')])
            );
        }

        $input['icon'] =  $request->hasFile('icon') ?$this->updateImage($typeLevel->icon,$request->file('icon'), 'typeLevels'):$typeLevel->icon;

        $typeLevel = $this->typeLevelRepository->update($input, $id);

        return $this->sendResponse(
            new TypeLevelResource($typeLevel),
            __('messages.updated', ['model' => __('models/typeLevels.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $typeLevel = $this->typeLevelRepository->find($id);
        $typeLevel->active = !$typeLevel->active;
        $typeLevel->save();

        return $this->sendResponse(
            new TypeLevelResource($typeLevel),
            __('messages.updated', ['model' => __('models/typeLevels.singular')])
        );
    }

    /**
     * Remove the specified TypeLevel from storage.
     * DELETE /typeLevels/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var TypeLevel $typeLevel */
        $typeLevel = $this->typeLevelRepository->find($id);

        if (empty($typeLevel)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/typeLevels.singular')])
            );
        }

        $typeLevel->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/typeLevels.singular')])
        );
    }
}
