<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateUserAPIRequest;
use App\Http\Requests\API\UpdateUserAPIRequest;
use App\Http\Resources\StudentResource;
use App\Models\Package;
use App\Models\Subscription;
use App\Models\User;
use App\Repositories\UserRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use App\Http\Resources\UserResource;
use Response;
use Validator;

/**
 * Class UserController
 * @package App\Http\Controllers\API
 */

class UserAPIController extends AppBaseController
{
    /** @var  UserRepository */
    private $userRepository;

    public function __construct(UserRepository $userRepo)
    {
        $this->userRepository = $userRepo;
    }

    /**
     * Display a listing of the User.
     * GET|HEAD /users
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        $users = $this->userRepository->all(
            $request->except(['skip', 'limit']),
            $request->get('skip'),
            $request->get('limit')
        )->where('role',$request->role);

        return $this->sendResponse(
            UserResource::collection($users),
            __('messages.retrieved', ['model' => __('models/users.plural')])
        );
    }

    /**
     * Store a newly created User in storage.
     * POST /users
     *
     * @param CreateUserAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(CreateUserAPIRequest $request)
    {
        $input = $request->all();

        if($request->file('logo') && $request->file('logo')->isValid()) {
        $input['logo'] = $this->saveImage($request->file('logo'), 'users');
        }

        $user = $this->userRepository->create($input);

        if($request->permissions){
            foreach($request->permissions as $permission){
                $user->givePermissionTo($permission);
            }
        }

        if($request->package_id){
            $package = Package::find(\request('package_id'));
            $subscription = new Subscription();
            $subscription->user_id = auth('sanctum')->id();
            $subscription->package_id = $package->id;
            $subscription->start_date = now();
            $subscription->end_date = $package->duration_type == 'month' ? now()->addMonths($package->duration) : ($package->duration_type == 'day' ? now()->addDays($package->duration) : now()->addYears($package->duration));
            $subscription->active = true;
            $subscription->cost = $package->price;
            $subscription->save();
        }


        return $this->sendResponse(
            new UserResource($user),
            __('messages.saved', ['model' => __('models/users.singular')])
        );
    }

    /**
     * Display the specified User.
     * GET|HEAD /users/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show($id)
    {
        /** @var User $user */
        $user = $this->userRepository->find($id);

        if (empty($user)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/users.singular')])
            );
        }

        return $this->sendResponse(
            new UserResource($user),
            __('messages.retrieved', ['model' => __('models/users.singular')])
        );
    }

    public function change_user_password($id , Request $request)
    {
        $validator = Validator::make($request->all(), [
            'new_password' => 'required|confirmed',
        ]);
        if ($validator->fails()) {
            return $this->sendError($validator->errors());
        }

        $user = $this->userRepository->find($id);


        if ($request->user()->role =="admin") {
            $user->password = bcrypt($request->new_password);
            $user->save();
            return $this->sendSuccess('Password changed Successfully');
        }

        return $this->sendError('You are not admin');

    }

    /**
     * Update the specified User in storage.
     * PUT/PATCH /users/{id}
     *
     * @param int $id
     * @param UpdateUserAPIRequest $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, UpdateUserAPIRequest $request)
    {
        $input = $request->all();

        /** @var User $user */
        $user = $this->userRepository->find($id);

        if (empty($user)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/users.singular')])
            );
        }

        $input['logo'] = $request->hasFile('logo') ?$this->updateImage($user->logo,$request->file('logo'), 'users'):$user->logo;
        $this->userRepository->update($input, $id);

        $user->permissions()->sync($request->permissions);

        return $this->sendResponse(
            new UserResource($user),
            __('messages.updated', ['model' => __('models/users.singular')])
        );
    }

    public function activateToggle($id): \Illuminate\Http\JsonResponse
    {
        $user = $this->userRepository->find($id);
        $user->active = !$user->active;
        $user->save();

        return $this->sendResponse(
            new UserResource($user),
            __('messages.updated', ['model' => __('models/users.singular')])
        );
    }

    /**
     * Remove the specified User from storage.
     * DELETE /users/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     *
     */
    public function destroy($id)
    {
        /** @var User $user */
        $user = $this->userRepository->find($id);

        if (empty($user)) {
            return $this->sendError(
                __('messages.not_found', ['model' => __('models/users.singular')])
            );
        }

        $user->delete();

        return $this->sendResponse(
            $id,
            __('messages.deleted', ['model' => __('models/users.singular')])
        );
    }

    public function multi_delete(Request $request)
    {
        foreach ($request->ids as $id) {
            $user = $this->userRepository->find($id);

            if (empty($user)) {
                return $this->sendError(
                    __('messages.not_found', ['model' => __('models/users.singular')])
                );
            }

            $user->delete();
        }

        return $this->sendResponse(
            $request->ids,
            __('messages.deleted', ['model' => __('models/users.singular')])
        );
    }
}
