<?php

namespace App\Models;

use App\Models\Models\CompetitionQuestion;
use Eloquent as Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Factories\HasFactory;

/**
 * Class Competition
 * @package App\Models
 * @version April 4, 2024, 12:28 am UTC
 *
 * @property integer $type_level_id
 * @property integer $level_category_id
 * @property integer $question_type_id
 * @property string $competition_datetime
 * @property string $question_speed
 * @property string $training_period
 * @property integer $numbers_count
 * @property integer $questions_count
 * @property string $answer_type
 * @property boolean $active
 * @property boolean $finish
 */
class Competition extends Model
{
    use SoftDeletes;

    use HasFactory;

    public $table = 'competitions';


    protected $dates = ['deleted_at'];


    protected $appends = ['end_time'];

    public function getEndTimeAttribute(){
        return $this->competition_datetime ? $this->competition_datetime->addMinutes($this->training_period) : null;
    }



    public $fillable = [
        'user_id',
        'type_level_id',
        'level_category_id',
        'question_type_id',
        'competition_datetime',
        'expired_at',
        'question_speed',
        'training_period',
        'questions_count_2num',
        'questions_count_3num',
        'questions_count_4num',
        'questions_count_5num',
        'questions_count_6num',
        'questions_count_7num',
        'questions_count_8num',
        'questions_count_9num',
        'questions_count_10num',
        'answer_type',
        'active',
        'finish'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'type_level_id' => 'integer',
        'level_category_id' => 'integer',
        'question_type_id' => 'integer',
        'competition_datetime' => 'datetime',
        'question_speed' => 'integer',
        'training_period' => 'integer',
        'questions_count_2num' => 'integer',
        'questions_count_3num' => 'integer',
        'questions_count_4num' => 'integer',
        'questions_count_5num' => 'integer',
        'questions_count_6num' => 'integer',
        'questions_count_7num' => 'integer',
        'questions_count_8num' => 'integer',
        'questions_count_9num' => 'integer',
        'questions_count_10num' => 'integer',
        'answer_type' => 'string',
        'active' => 'boolean',
        'finish' => 'boolean'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'type_level_id' => 'exists:type_levels,id',
        'level_category_id' => 'exists:level_categories,id',
        'question_type_id' => 'exists:question_types,id',
        'competition_datetime' => 'required',
        'question_speed' => 'required',
        'training_period' => 'required',
//        'numbers_count' => 'required|numeric',
//        'questions_count' => 'required|numeric',
        'answer_type' => 'required'
    ];

    public function subscribers (){
        return $this->hasMany(CompetitionSubscriber::class)->orderByDesc('supposed_solved')->orderBy('student_points','desc')->orderBy('remaining_time','desc')->orderBy('answering_period','asc');
    }

    public function my_subscriber(){
        return $this->hasOne(CompetitionSubscriber::class)->where('user_id',auth('sanctum')->id());
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function type_level(): BelongsTo
    {
        return $this->belongsTo(TypeLevel::class);
    }

    public function level_category(): BelongsTo
    {
        return $this->belongsTo(LevelCategory::class);
    }

    public function question_type(): BelongsTo
    {
        return $this->belongsTo(QuestionType::class);
    }

    public function questions(): BelongsToMany
    {
        return $this->belongsToMany(Question::class, 'competition_questions')->inRandomOrder();
    }

    public function scopeActive($query)
    {
        return $query->where('active',1)->whereHas('questions');
    }

    public function scopeFinished($query)
    {
        if($this->competition_datetime == null)
           return $query->where('finish',1);
        else
           return $query->whereDate('competition_datetime','<=' ,now()->subMinutes($this->training_period))->orwhere('finish',1);
    }

    public function scopeMustFinished($query)
    {
           return $query->where('competition_datetime','<' ,now()->subMinutes($this->training_period)->timezone('Africa/Cairo'))->where('finish',0);
    }


}
