<?php

namespace App\Models;

use App\Models\Models\TrainingQuestion;
use Eloquent as Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Factories\HasFactory;

/**
 * Class Training
 * @package App\Models
 * @version March 28, 2024, 10:48 pm UTC
 *
 * @property integer $training_type_id
 * @property integer $type_level_id
 * @property integer $level_category_id
 * @property integer $question_type_id
 * @property string $question_speed
 * @property string $training_period
 * @property integer $numbers_count
 * @property integer $questions_count
 * @property string $answer_type
 * @property integer $correct_answers_count
 * @property integer $wrong_answers_count
 * @property string $answering_period
 * @property integer $student_points
 * @property boolean $active
 * @property boolean $finish
 */
class Training extends Model
{
    use SoftDeletes;

    use HasFactory;

    public $table = 'trainings';


    protected $dates = ['deleted_at'];

    protected $appends = ['end_time'];


    public function getEndTimeAttribute()
    {
        return $this->training_datetime ? $this->training_datetime->addMinutes($this->training_period) : null;
    }

    public $end_time ;



    public $fillable = [
        'user_id',
        'training_type_id',
        'type_level_id',
        'level_category_id',
        'question_type_id',
        'question_speed',
        'training_period',
        'questions_count_2num',
        'questions_count_3num',
        'questions_count_4num',
        'questions_count_5num',
        'questions_count_6num',
        'questions_count_7num',
        'questions_count_8num',
        'questions_count_9num',
        'questions_count_10num',
        'answer_type',
        'correct_answers_count',
        'wrong_answers_count',
        'answering_period',
        'remaining_time',
        'student_points',
        'supposed_solved',
        'active',
        'training_datetime',
        'expired_at',
        'trainer_id',
        'finish',
        'training_id',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'training_type_id' => 'integer',
        'type_level_id' => 'integer',
        'level_category_id' => 'integer',
        'question_type_id' => 'integer',
        'question_speed' => 'integer',
        'training_period' => 'integer',
        'questions_count_2num' => 'integer',
        'questions_count_3num' => 'integer',
        'questions_count_4num' => 'integer',
        'questions_count_5num' => 'integer',
        'questions_count_6num' => 'integer',
        'questions_count_7num' => 'integer',
        'questions_count_8num' => 'integer',
        'questions_count_9num' => 'integer',
        'questions_count_10num' => 'integer',
        'answer_type' => 'string',
        'correct_answers_count' => 'integer',
        'wrong_answers_count' => 'integer',
        'answering_period' => 'string',
        'student_points' => 'integer',
        'active' => 'boolean',
        'finish' => 'boolean',
        'training_datetime' => 'datetime'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'training_type_id' => 'exists:training_types,id',
        'type_level_id' => 'exists:type_levels,id',
        'level_category_id' => 'exists:level_categories,id',
        'question_type_id' => 'exists:question_types,id',
        'training_period' => 'required',
//        'numbers_count' => 'required|numeric',
//        'questions_count' => 'required|numeric',
        'answer_type' => 'required',
        'correct_answers_count' => 'numeric',
        'wrong_answers_count' => 'numeric',
        'student_points' => 'numeric'
    ];

    public function training_type(): BelongsTo
    {
        return $this->belongsTo(TrainingType::class);
    }

    public function type_level(): BelongsTo
    {
        return $this->belongsTo(TypeLevel::class);
    }

    public function level_category(): BelongsTo
    {
        return $this->belongsTo(LevelCategory::class);
    }

    public function question_type(): BelongsTo
    {
        return $this->belongsTo(QuestionType::class);
    }

    public function trainer()
    {
        return $this->belongsTo(User::class,'trainer_id');
    }

    public function user()
    {
        return $this->belongsTo(User::class,'user_id');
    }

    public function questions(): BelongsToMany
    {
        return $this->belongsToMany(Question::class, 'training_questions')->inRandomOrder();
    }

    public function training_questions(): HasMany
    {
        return $this->hasMany(TrainingQuestion::class);
    }

    public function subscribers (){
        return $this->hasMany(Training::class);
    }

    public function scopeActive($query)
    {
        return $query->where('active',1)->whereHas('questions');
    }

    public function scopeFinished($query)
    {
        if($this->training_datetime == null)
           return $query->where('finish',1);
        else
           return $query->whereDate('training_datetime','<=' ,now()->subMinutes($this->training_period))->orwhere('finish',1);
    }


    public function scopeMustFinished($query)
    {
           return $query->where('training_datetime','<' ,now()->subMinutes($this->training_period)->timezone('Africa/Cairo'))->where('finish',0);
    }


}
