<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
// use Illuminate\Database\Eloquent\SoftDeletes;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Validation\Rules\Enum;
use Laravel\Sanctum\HasApiTokens;
use Spatie\Permission\Traits\HasRoles;


/**
 * Class User
 * @package App\Models
 * @version March 23, 2024, 1:13 pm UTC
 *
 * @property string $name
 * @property string $phone
 * @property string $business_phone
 * @property string $email
 * @property enum $gender
 * @property integer $country_id
 * @property integer $city_id
 * @property integer $region_id
 * @property string $password
 * @property string $birth_date
 * @property integer $otp
 * @property enum $role
 */
class User extends Authenticatable
{
    use HasApiTokens, Notifiable;
    // use SoftDeletes;
    use HasRoles;

    public $table = 'users';
    protected $dates = ['deleted_at'];

    protected $hidden = [
        'password',
        'remember_token',
    ];

    public $fillable = [
        'name',
        'phone',
        'business_phone',
        'email',
        'gender',
        'country_id',
        'city_id',
        'region_id',
        'password',
        'birth_date',
        'logo',
        'profile_id',
        'provide_name',
        'otp',
        'role',
        'special_trainer',
        'country_code'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'name' => 'string',
        'phone' => 'string',
        'business_phone' => 'string',
        'email' => 'string',
        'country_id' => 'integer',
        'city_id' => 'integer',
        'region_id' => 'integer',
        'phone_verified_at' => 'datetime',
        'email_verified_at' => 'datetime',
        'password' => 'hashed',
        'birth_date' => 'date',
        'otp' => 'integer'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'name' => 'required|max:50',
        'phone' => 'required|string|unique:users,phone,NULL,id,deleted_at,NULL|numeric',
        'email' => 'nullable|unique:users,email,NULL,id,deleted_at,NULL|email|max:255',
        'country_id' => 'exists:countries,id',
        'city_id' => 'exists:cities,id',
        'region_id' => 'exists:regions,id'
    ];

    public static $update_rules = [
        'name' => 'sometimes|max:50',
        'phone' => 'sometimes|numeric',
        'email' => 'sometimes|email|max:255',
        'country_id' => 'sometimes|exists:countries,id',
        'city_id' => 'sometimes|exists:cities,id',
        'region_id' => 'sometimes|exists:regions,id'
    ];

     protected $appends = ['student_level','age_category_id','age_category_title','age'];

     public function getStudentLevelAttribute()
    {
        if ($this->role =='student' && $this->student_points <> null) {
                $point_level = PointLevel::where('from','<=',$this->student_points)->where('to','>=',$this->student_points)->first();
                return $point_level?$point_level->level:null;
        }
        return null;
    }

    public function getAgeAttribute()
    {
        return Carbon::parse($this->attributes['birth_date'])->age;
    }

    public function getAgeCategoryIdAttribute()
    {
        if ($this->role =='student' && $this->age <> null) {
            if($this->age <5 ){
                return 1;
            }elseif($this->age >=5 &&$this->age <8 ){
                return 2;
            }
            elseif($this->age >=8 && $this->age <=10 ){
                return 3;
            }
            elseif($this->age >10 ){
                return 4;
            }
        }
        return null;
    }

    public function getAgeCategoryTitleAttribute()
    {
        if ($this->role =='student' && $this->age <> null) {
            if($this->age <5 ){
                return AgeCategory::find(1)->title;
            }elseif($this->age >=5 &&$this->age <8 ){
                return AgeCategory::find(2)->title;
            }
            elseif($this->age >=8 && $this->age <=10 ){
                return AgeCategory::find(3)->title;
            }
            elseif($this->age >10 ){
                return AgeCategory::find(4)->title;
            }
        }
        return null;
    }



    public function subscription(): HasOne
    {
        $now = Carbon::now();
        return $this->hasOne(Subscription::class)->whereDate('end_date','>',$now);
    }

    public function friends(): BelongsToMany
    {
        return $this->belongsToMany(User::class,'favourites','user_id','friend_id')->distinct()->where('role','student')->where('favourites.active',1);
    }

    // public function friendsFrom(): BelongsToMany
    // {
    //     return $this->belongsToMany(User::class,'favourites','friend_id','user_id')->distinct()->where('favourites.active',1);
    // }
    // public function friends()
    // {
    //     return $this->friendsTo()->union($this->friendsFrom());
    // }

    public function reviews(): HasMany
    {
        return $this->hasMany(TrainerReview::class,'trainer_id')->orderBy('rate','desc');
    }

    public function levels(): BelongsToMany
    {
        return $this->belongsToMany(TypeLevel::class, 'user_levels');
    }

    public function trainings(): HasMany
    {
        return $this->hasMany(Training::class);
    }

    public function country(): BelongsTo
    {
        return $this->belongsTo(Country::class,'country_id');
    }
    public function region(): BelongsTo
    {
        return $this->belongsTo(Region::class,'region_id');
    }
    public function city(): BelongsTo
    {
        return $this->belongsTo(City::class,'city_id');
    }

    public function winners_subscribers(){
        return $this->hasMany(ChampionshipSubscriber::class)->whereNotNull('order')->orderBy('student_points','desc')->orderBy('updated_at','desc');
    }

    public function trainer_regions(): HasMany
    {
        return $this->hasMany(TrainerRegion::class);
    }



}
