<?php

namespace App\Repositories;

use App\Models\Question;
use Illuminate\Container\Container as Application;
use Illuminate\Database\Eloquent\Model;
use function is_numeric;
use Illuminate\Support\Facades\Schema;

abstract class BaseRepository
{
    /**
     * @var Model
     */
    protected $model;

    /**
     * @var Application
     */
    protected $app;

    /**
     * @param Application $app
     *
     * @throws \Exception
     */
    public function __construct(Application $app)
    {
        $this->app = $app;
        $this->makeModel();
    }

    /**
     * Get searchable fields array
     *
     * @return array
     */
    abstract public function getFieldsSearchable();

    /**
     * Configure the Model
     *
     * @return string
     */
    abstract public function model();

    /**
     * Make Model instance
     *
     * @throws \Exception
     *
     * @return Model
     */
    public function makeModel()
    {
        $model = $this->app->make($this->model());

        if (!$model instanceof Model) {
            throw new \Exception("Class {$this->model()} must be an instance of Illuminate\\Database\\Eloquent\\Model");
        }

        return $this->model = $model;
    }

    /**
     * Paginate records for scaffold.
     *
     * @param int $perPage
     * @param array $columns
     * @return \Illuminate\Contracts\Pagination\LengthAwarePaginator
     */
    public function paginate($perPage, $columns = ['*'])
    {
        $query = $this->allQuery();

        return $query->paginate($perPage, $columns);
    }

    /**
     * Build a query for retrieving all records.
     *
     * @param array $search
     * @param int|null $skip
     * @param int|null $limit
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function allQuery($search = [], $skip = null, $limit = null)
    {
        $query = $this->model->newQuery();

        if (count($search)) {
            foreach($search as $key => $value) {
                if (in_array($key, $this->getFieldsSearchable())) {
                    $query->where($key, $value);
                }
            }
        }

        if (!is_null($skip)) {
            $query->skip($skip);
        }

        if (!is_null($limit)) {
            $query->limit($limit);
        }

        return $query;
    }

    /**
     * Retrieve all records with given filter criteria
     *
     * @param array $search
     * @param int|null $skip
     * @param int|null $limit
     * @param array $columns
     *
     * @return \Illuminate\Contracts\Pagination\LengthAwarePaginator|\Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection
     */
    public function all($search = [], $skip = null, $limit = null, $columns = ['*'])
    {
        $query = $this->allQuery($search, $skip, $limit);

        if (auth('sanctum')->check()) {
            if (auth('sanctum')->user()->role =='student' || auth('sanctum')->user()->role == 'trainer') {
                if (Schema::hasColumn($this->model->getTable(), 'active')){
                    $query = $query->whereNotNull('active')->where('active',1);
                }
            }
        }

        if (request('rows') && request('rows') != 'all') {
            return $query->paginate(request('rows'));
        }

        return $query->get($columns);
    }

    /**
     * Create model record
     *
     * @param array $input
     *
     * @return Model
     */
    public function create($input)
    {
        $model = $this->model->newInstance($input);

        $model->save();

        return $model;
    }

    /**
     * Find model record for given id
     *
     * @param int $id
     * @param array $columns
     *
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection|Model|null
     */
    public function find($id, $columns = ['*'])
    {
        $query = $this->model->newQuery();

//        if(is_numeric($id)){
//            return $query->find($id, $columns);
//        }
//
//        return $query->where('slug',$id)->first();
        return $query->find($id, $columns);
    }

    /**
     * Update model record for given id
     *
     * @param array $input
     * @param int $id
     *
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection|Model
     */
    public function update($input, $id)
    {
        $query = $this->model->newQuery();

        $model = $query->findOrFail($id);

        $model->fill($input);

        $model->save();

        return $model;
    }

    /**
     * @param int $id
     *
     * @throws \Exception
     *
     * @return bool|mixed|null
     */
    public function delete($id)
    {
        $query = $this->model->newQuery();

        $model = $query->findOrFail($id);

        return $model->delete();
    }


public function get_questions($training)
{
    $questions = Question::where('active', 1)
        ->where('type_level_id', $training->type_level_id);

    if ($training->level_category_id !== null) {
        $questions = $questions->where('level_category_id', $training->level_category_id);
    }

    $questions = $questions->inRandomOrder()->get();

    $all_questions = collect();
    for ($i = 2; $i <= 10; $i++) {
        $questions_count_key = "questions_count_{$i}num";
        $requested_count = $training->$questions_count_key ?? 0;

        if ($requested_count > 0) {
            $filtered_questions = $questions->where('numbers_count', $i)->take($requested_count);
            
            if ($filtered_questions->count() < $requested_count) {
                $training->delete();
                return response()->json([
                    "message" => trans("messages.Not_enough_questions_for{$i}numbers") . $filtered_questions->count(),
                    "status" => 404,
                ], 404);
            }

            $all_questions = $all_questions->merge($filtered_questions);
        }
    }

    if ($all_questions->isEmpty()) {
        $training->delete();
        return response()->json([
            "message" => trans('messages.NoQuestionsAvailable_for_this_setting'),
            "status" => 404,
        ], 404);
    }

    return $all_questions;
}

}
